import 'package:fish_redux/fish_redux.dart';
import 'package:flutter_livepush_demo/models/music_info.dart';
import 'package:flutter_livepush_demo/page/camera_push/camera_push_state.dart';

enum CameraPushAction {
  previewCreated,
  clickPreview,
  onClickPreview,
  clickePush,
  onClickPush,
  clickPausPush,
  onClickPausePush,
  clickReStartPush,
  showBeauty,
  hideBeauty,
  onChangeBeautyPanel,
  clickFlashLamp,
  onClickFlashLamp,
  clickSwitchCamera,
  clickSnapShot,
  goBackPage,
  updatePushStatusTip,
}

enum CameraPushBGMAction {
  changeEars,
  onChangeEars,
  changeAudioDenoise,
  onChangeAudioDenoise,
  changeMute,
  onChangeMute,
  changePlayState,
  onChangePlayState,
  changeLoop,
  onChangeLoop,
  changeBGMVolume,
  onChangeBGMVolume,
  changeCaptureVolume,
  onChangeCaptureVolume,
  changeBGMSource,
  onChangeBGMSource,
  updateBGMProgress,
  onUpdateBGMProgress,
  updateBGMDuration,
  onUpdateBGMDuration,
}

enum CameraPushConfigAction {
  changePushMirror,
  onChangePushMirror,
  changePreviewMirror,
  onChangePreviewMirror,
  setPreviewDisplayMode,
  onSetPreviewDisplayMode,
  setTargetVideoBitrate,
  onSetTargetVideoBitrate,
  setMinVideoBitrate,
  onSetMinVideoBitrate
}

enum CameraPushBottomSheetAction {
  showMusic,
  onShowMusic,
  hideMusic,
  onHideMusic,
  showMoreSetting,
  onShowMoreSetting,
  hideMoreSetting,
  onHideMoreSetting,
}

class CameraPushActionCreator {
  //preview
  static Action previewCreated() =>
      const Action(CameraPushAction.previewCreated);
  static Action clickPreview() => const Action(CameraPushAction.clickPreview);
  static Action onClickPreview(CameraPushPagePreviewState previewState) =>
      Action(CameraPushAction.onClickPreview, payload: previewState);

  //push
  static Action clickPush(String pushURL) =>
      Action(CameraPushAction.clickePush, payload: pushURL);
  static Action onClickPush(CameraPushPagePushState pushState) =>
      Action(CameraPushAction.onClickPush, payload: pushState);
  static Action clickPausPush() => const Action(CameraPushAction.clickPausPush);
  static Action onClickPausePush(CameraPushPagePushState pushState) =>
      Action(CameraPushAction.onClickPausePush, payload: pushState);
  static Action clickReStartPush() =>
      const Action(CameraPushAction.clickReStartPush);

  static Action showBeauty() => const Action(CameraPushAction.showBeauty);
  static Action hideBeauty() => const Action(CameraPushAction.hideBeauty);
  static Action onChangeBeautyPanel(bool isShow) => Action(CameraPushAction.onChangeBeautyPanel, payload: isShow);
  static Action clickFlashLamp(bool isOpen) =>
      Action(CameraPushAction.clickFlashLamp, payload: isOpen);
  static Action onClickFlashLamp(bool isOpen) =>
      Action(CameraPushAction.onClickFlashLamp, payload: isOpen);
  static Action clickSwitchCamera() =>
      const Action(CameraPushAction.clickSwitchCamera);
  static Action clickSnapShot() => const Action(CameraPushAction.clickSnapShot);
  static Action goBackPage() => const Action(CameraPushAction.goBackPage);
  static Action updatePushStatusTip(String tip) =>
      Action(CameraPushAction.updatePushStatusTip, payload: tip);

  //BGM
  static Action changeEars(bool isOpen) =>
      Action(CameraPushBGMAction.changeEars, payload: isOpen);
  static Action onChangeEars(bool isOpen) =>
      Action(CameraPushBGMAction.onChangeEars, payload: isOpen);
  static Action changeAudioDenoise(bool isOpen) =>
      Action(CameraPushBGMAction.changeAudioDenoise, payload: isOpen);
  static Action onChangeAudioDenoise(bool isOpen) =>
      Action(CameraPushBGMAction.onChangeAudioDenoise, payload: isOpen);
  static Action changeMute(bool isOpen) =>
      Action(CameraPushBGMAction.changeMute, payload: isOpen);
  static Action onChangeMute(bool isOpen) =>
      Action(CameraPushBGMAction.onChangeMute, payload: isOpen);
  static Action changePlayState(bool isPlaying) =>
      Action(CameraPushBGMAction.changePlayState, payload: isPlaying);
  static Action onChangePlayState(bool isPlaying) =>
      Action(CameraPushBGMAction.onChangePlayState, payload: isPlaying);
  static Action changeLoop(bool isOpen) =>
      Action(CameraPushBGMAction.changeLoop, payload: isOpen);
  static Action onChangeLoop(bool isOpen) =>
      Action(CameraPushBGMAction.onChangeLoop, payload: isOpen);
  static Action changeBGMVolume(int bgmVolume) =>
      Action(CameraPushBGMAction.changeBGMVolume, payload: bgmVolume);
  static Action onChangeBGMVolume(int bgmVolume) =>
      Action(CameraPushBGMAction.onChangeBGMVolume, payload: bgmVolume);
  static Action changeCaptureVolume(int captureVolume) =>
      Action(CameraPushBGMAction.changeCaptureVolume, payload: captureVolume);
  static Action onChangeCaptureVolume(int captureVolume) =>
      Action(CameraPushBGMAction.onChangeCaptureVolume, payload: captureVolume);
  static Action changeBGMSource(MusicInfo musicInfo) =>
      Action(CameraPushBGMAction.changeBGMSource, payload: musicInfo);
  static Action onChangeBGMSource(MusicInfo musicInfo) =>
      Action(CameraPushBGMAction.onChangeBGMSource, payload: musicInfo);
  static Action updateBGMProgress(int progress) =>
      Action(CameraPushBGMAction.updateBGMProgress, payload: progress);
  static Action onUpdateBGMProgress(int progress) =>
      Action(CameraPushBGMAction.onUpdateBGMProgress, payload: progress);
  static Action updateBGMDuration(int duration) =>
      Action(CameraPushBGMAction.updateBGMDuration, payload: duration);
  static Action onUpdateBGMDuration(int duration) =>
      Action(CameraPushBGMAction.onUpdateBGMDuration, payload: duration);

  //more config
  static Action changePushMirror(bool isMirror) =>
      Action(CameraPushConfigAction.changePushMirror, payload: isMirror);
  static Action onChangePushMirror(bool isMirror) =>
      Action(CameraPushConfigAction.onChangePushMirror, payload: isMirror);
  static Action changePreviewMirror(bool isPreview) =>
      Action(CameraPushConfigAction.changePreviewMirror, payload: isPreview);
  static Action onChangePreviewMirror(bool isPreview) =>
      Action(CameraPushConfigAction.onChangePreviewMirror, payload: isPreview);
  static Action setPreviewDisplayMode(newValue) =>
      Action(CameraPushConfigAction.setPreviewDisplayMode, payload: newValue);
  static Action onSetPreviewDisplayMode(newValue) =>
      Action(CameraPushConfigAction.onSetPreviewDisplayMode, payload: newValue);
  static Action setTargetVideoBitrate(newValue) =>
      Action(CameraPushConfigAction.setTargetVideoBitrate, payload: newValue);
  static Action onSetTargetVideoBitrate(newValue) =>
      Action(CameraPushConfigAction.onSetTargetVideoBitrate, payload: newValue);
  static Action setMinVideoBitrate(newValue) =>
      Action(CameraPushConfigAction.setMinVideoBitrate, payload: newValue);
  static Action onSetMinVideoBitrate(newValue) =>
      Action(CameraPushConfigAction.onSetMinVideoBitrate, payload: newValue);

  //bottom sheet
  static Action showMusic() =>
      const Action(CameraPushBottomSheetAction.showMusic);
  static Action onShowMusic() =>
      const Action(CameraPushBottomSheetAction.onShowMusic);
  static Action hideMusic() =>
      const Action(CameraPushBottomSheetAction.hideMusic);
  static Action onHideMusic() =>
      const Action(CameraPushBottomSheetAction.onHideMusic);
  static Action showMoreSetting() =>
      const Action(CameraPushBottomSheetAction.showMoreSetting);
  static Action onShowMoreSetting() =>
      const Action(CameraPushBottomSheetAction.onShowMoreSetting);
  static Action hideMoreSetting() =>
      const Action(CameraPushBottomSheetAction.hideMoreSetting);
  static Action onHideMoreSetting() =>
      const Action(CameraPushBottomSheetAction.onHideMoreSetting);
}
