import 'dart:io';

import 'package:fish_redux/fish_redux.dart';

import 'package:flutter/cupertino.dart' hide Action;
import 'package:flutter/services.dart';
import 'package:flutter/widgets.dart' hide Action;
import 'package:flutter_livepush_demo/common/CommonUtils.dart';
import 'package:flutter_livepush_demo/models/music_info.dart';
import 'package:flutter_livepush_demo/page/camera_push/camera_push_action.dart';
import 'package:flutter_livepush_demo/page/camera_push/camera_push_state.dart';
import 'package:flutter_livepush_plugin/live_base.dart';
import 'package:flutter_livepush_plugin/live_pusher.dart';
import 'package:flutter_livepush_plugin/live_push_def.dart';
import 'package:flutter_livepush_plugin/beauty/live_beauty.dart';
import 'package:fluttertoast/fluttertoast.dart';
import 'package:flutter_gen/gen_l10n/app_localizations.dart';

Effect<CameraPushState> buildEffect() {
  return combineEffects(<Object, Effect<CameraPushState>>{
    Lifecycle.initState: _init,
    Lifecycle.dispose: _dispose,
    CameraPushAction.showBeauty: _showBeauty,
    CameraPushAction.hideBeauty: _hideBeauty,
    CameraPushAction.clickFlashLamp: _clickFlashLamp,
    CameraPushAction.clickSwitchCamera: _clickSwitchCamera,
    CameraPushAction.clickSnapShot: _clickSnapShot,
    CameraPushAction.previewCreated: _previewCreated,
    CameraPushAction.clickPreview: _clickPreview,
    CameraPushAction.clickePush: _clickPushAction,
    CameraPushAction.clickPausPush: _clickPausPush,
    CameraPushAction.clickReStartPush: _clickReStartPush,
    CameraPushAction.goBackPage: _goBackPage,
    //bgm
    CameraPushBGMAction.changeEars: _changeEars,
    CameraPushBGMAction.changeAudioDenoise: _changeAudioDenoise,
    CameraPushBGMAction.changeMute: _changeMute,
    CameraPushBGMAction.changePlayState: _changePlayState,
    CameraPushBGMAction.changeLoop: _changeLoop,
    CameraPushBGMAction.changeBGMVolume: _changeBGMVolume,
    CameraPushBGMAction.changeCaptureVolume: _changeCaptureVolume,
    CameraPushBGMAction.changeBGMSource: _changeBGMSource,
    //config
    CameraPushConfigAction.changePushMirror: _changePushMirror,
    CameraPushConfigAction.changePreviewMirror: _changePreviewMirror,
    CameraPushConfigAction.setPreviewDisplayMode: _setPreviewDisplayMode,
    CameraPushConfigAction.setTargetVideoBitrate: _setTargetVideoBitrate,
    CameraPushConfigAction.setMinVideoBitrate: _setMinVideoBitrate,
    //page state
    CameraPushBottomSheetAction.showMusic: _showMusic,
    CameraPushBottomSheetAction.hideMusic: _hideMusic,
    CameraPushBottomSheetAction.showMoreSetting: _showMoreSetting,
    CameraPushBottomSheetAction.hideMoreSetting: _hideMoreSetting,
  });
}

late AlivcBase _alivcBase;
late AlivcLivePusher _alivcLivePusher;
late AlivcLiveBeautyManager _beautyManager;

Future<void> _init(Action action, Context<CameraPushState> ctx) async {
  _setPageOrientation(action, ctx);
  _alivcBase = AlivcBase.init();
  _alivcBase.registerSDK();
  _alivcBase.setObserver();
  _setLivePusher(action, ctx);
  _onListen(action, ctx);
}

Future<void> _onListen(Action action, Context<CameraPushState> ctx) async {
  _alivcBase.setOnLicenceCheck((result, reason) {
    if (result != AlivcLiveLicenseCheckResultCode.success) {
      Fluttertoast.showToast(
        msg: reason,
        gravity: ToastGravity.CENTER,
      );
    }
  });

  _alivcLivePusher.setOnSDKError((errorCode, errorDescription) {
    Fluttertoast.showToast(
      msg: AppLocalizations.of(ctx.context)!.camerapush_sdk_error,
      gravity: ToastGravity.CENTER,
    );
  });

  /// 系统错误回调
  _alivcLivePusher.setOnSystemError((errorCode, errorDescription) {
    Fluttertoast.showToast(
      msg: AppLocalizations.of(ctx.context)!.camerapush_system_error,
      gravity: ToastGravity.CENTER,
    );
  });

  /// 开始预览回调
  _alivcLivePusher.setOnPreviewStarted(() {
    ctx.dispatch(CameraPushActionCreator.updatePushStatusTip(
        AppLocalizations.of(ctx.context)!.camerapush_start_preview_log));
  });

  /// 停止预览回调
  _alivcLivePusher.setOnPreviewStoped(() {
    ctx.dispatch(CameraPushActionCreator.updatePushStatusTip(
        AppLocalizations.of(ctx.context)!.camerapush_stop_preview_log));
  });

  /// 渲染第一帧回调
  _alivcLivePusher.setOnFirstFramePreviewed(() {
    ctx.dispatch(CameraPushActionCreator.updatePushStatusTip(
        AppLocalizations.of(ctx.context)!.camerapush_first_frame_log));
  });

  /// 推流开始回调
  _alivcLivePusher.setOnPushStarted(() {
    ctx.dispatch(CameraPushActionCreator.updatePushStatusTip(
        AppLocalizations.of(ctx.context)!.camerapush_start_push_log));
  });

  /// 摄像头推流暂停回调
  _alivcLivePusher.setOnPushPaused(() {
    ctx.dispatch(CameraPushActionCreator.updatePushStatusTip(
        AppLocalizations.of(ctx.context)!.camerapush_pause_push_log));
  });

  /// 摄像头推流恢复回调
  _alivcLivePusher.setOnPushResumed(() {
    ctx.dispatch(CameraPushActionCreator.updatePushStatusTip(
        AppLocalizations.of(ctx.context)!.camerapush_resume_push_log));
  });

  /// 重新推流回调
  _alivcLivePusher.setOnPushRestart(() {
    ctx.dispatch(CameraPushActionCreator.updatePushStatusTip(
        AppLocalizations.of(ctx.context)!.camerapush_restart_push_log));
  });

  /// 推流停止回调
  _alivcLivePusher.setOnPushStoped(() {
    ctx.dispatch(CameraPushActionCreator.updatePushStatusTip(
        AppLocalizations.of(ctx.context)!.camerapush_stop_push_log));
  });

  /// 推流链接失败
  _alivcLivePusher.setOnConnectFail((errorCode, errorDescription) {
    Fluttertoast.showToast(
      msg: AppLocalizations.of(ctx.context)!.camerapush_connect_failed_log,
      gravity: ToastGravity.CENTER,
    );
  });

  /// 网络恢复
  _alivcLivePusher.setOnConnectRecovery(() {
    ctx.dispatch(CameraPushActionCreator.updatePushStatusTip(
        AppLocalizations.of(ctx.context)!.camerapush_connect_recovery_log));
  });

  /// 连接被断开
  _alivcLivePusher.setOnConnectionLost(() {
    Fluttertoast.showToast(
      msg: AppLocalizations.of(ctx.context)!.camerapush_connection_lost_log,
      gravity: ToastGravity.CENTER,
    );
  });

  /// 网络差回调
  _alivcLivePusher.setOnNetworkPoor(() {
    Fluttertoast.showToast(
      msg: AppLocalizations.of(ctx.context)!.camerapush_network_slow_log,
      gravity: ToastGravity.CENTER,
    );
  });

  /// 重连失败回调
  _alivcLivePusher.setOnReconnectError((errorCode, errorDescription) {
    Fluttertoast.showToast(
      msg: AppLocalizations.of(ctx.context)!.camerapush_reconnect_fail_log,
      gravity: ToastGravity.CENTER,
    );
  });

  /// 重连开始回调
  _alivcLivePusher.setOnReconnectStart(() {
    ctx.dispatch(CameraPushActionCreator.updatePushStatusTip(
        AppLocalizations.of(ctx.context)!.camerapush_reconnect_start_log));
  });

  /// 重连成功回调
  _alivcLivePusher.setOnReconnectSuccess(() {
    ctx.dispatch(CameraPushActionCreator.updatePushStatusTip(
        AppLocalizations.of(ctx.context)!.camerapush_reconnect_success_log));
  });

  /// 发送数据超时
  _alivcLivePusher.setOnSendDataTimeout(() {
    ctx.dispatch(CameraPushActionCreator.updatePushStatusTip(
        AppLocalizations.of(ctx.context)!.camerapush_send_data_timeout_log));
  });

  /// 背景音乐播放完毕
  _alivcLivePusher.setOnBGMCompleted(() {
    ctx.dispatch(CameraPushActionCreator.updatePushStatusTip(
        AppLocalizations.of(ctx.context)!.camerapush_bgm_play_complete_log));
  });

  /// 背景音乐下载播放超时
  _alivcLivePusher.setOnBGMDownloadTimeout(() {
    Fluttertoast.showToast(
      msg:
          AppLocalizations.of(ctx.context)!.camerapush_bgm_download_timeout_log,
      gravity: ToastGravity.CENTER,
    );
  });

  /// 背景音乐开启失败
  _alivcLivePusher.setOnBGMOpenFailed(() {
    Fluttertoast.showToast(
      msg:
          AppLocalizations.of(ctx.context)!.camerapush_bgm_file_open_failed_log,
      gravity: ToastGravity.CENTER,
    );
  });

  /// 背景音乐暂停播放
  _alivcLivePusher.setOnBGMPaused(() {
    ctx.dispatch(CameraPushActionCreator.updatePushStatusTip(
        AppLocalizations.of(ctx.context)!.camerapush_bgm_pause_log));
  });

  /// 背景音乐当前播放进度
  _alivcLivePusher.setOnBGMProgress((progress, duration) {
    ctx.dispatch(CameraPushActionCreator.onUpdateBGMProgress(progress));
    ctx.dispatch(CameraPushActionCreator.onUpdateBGMDuration(duration));
  });

  /// 背景音乐恢复播放
  _alivcLivePusher.setOnBGMResumed(() {
    ctx.dispatch(CameraPushActionCreator.updatePushStatusTip(
        AppLocalizations.of(ctx.context)!.camerapush_bgm_resume_log));
  });

  /// 背景音乐开始播放
  _alivcLivePusher.setOnBGMStarted(() {
    ctx.dispatch(CameraPushActionCreator.updatePushStatusTip(
        AppLocalizations.of(ctx.context)!.camerapush_bgm_start_log));
  });

  /// 背景音乐停止播放
  _alivcLivePusher.setOnBGMStoped(() {
    ctx.dispatch(CameraPushActionCreator.updatePushStatusTip(
        AppLocalizations.of(ctx.context)!.camerapush_bgm_stop_log));
    ctx.dispatch(CameraPushActionCreator.onUpdateBGMProgress(0));
    ctx.dispatch(CameraPushActionCreator.onUpdateBGMDuration(0));
  });

  /// 截图回调
  _alivcLivePusher.setOnSnapshot((saveResult, savePath, {dirTypeForIOS}) {
    if (saveResult == true) {
      String tip = AppLocalizations.of(ctx.context)!.camerapush_snapshot_tip;
      if (Platform.isIOS) {
        DirType saveDirType = DirType.document;
        if (dirTypeForIOS == AlivcLiveSnapshotDirType.document) {
          saveDirType = DirType.document;
        } else {
          saveDirType = DirType.library;
        }
        CommomUtils.getSaveDir(saveDirType, savePath).then((value) {
          Fluttertoast.showToast(
              msg: tip + value.path, gravity: ToastGravity.CENTER);
        });
      } else {
        Fluttertoast.showToast(
            msg: tip + savePath, gravity: ToastGravity.CENTER);
      }
    }
  });
}

Future<void> _setLivePusher(Action action, Context<CameraPushState> ctx) async {
  _alivcLivePusher = AlivcLivePusher.init();
  _alivcLivePusher.initLivePusher();
  _alivcLivePusher.setErrorDelegate();
  _alivcLivePusher.setInfoDelegate();
  _alivcLivePusher.setNetworkDelegate();
  _alivcLivePusher.setCustomFilterDelegate();
  _alivcLivePusher.setCustomDetectorDelegate();
  _alivcLivePusher.setBGMDelegate();

  if (ctx.state.beautyOn) {
    _setBeauty(action, ctx);
  }
}

Future<void> _setBeauty(Action action, Context<CameraPushState> ctx) async {
  _alivcLivePusher.useBeauty();
  _beautyManager = AlivcLiveBeautyManager.init();
  String sdkVersion = await AlivcLiveBeautyManager.getQueenSDKVersion();
  print("queen sdk version:${sdkVersion}");
}

Future<void> _previewCreated(
    Action action, Context<CameraPushState> ctx) async {
  _alivcLivePusher.startPreview();
  _beautyManager.setupBeauty();
  ctx.dispatch(CameraPushActionCreator.onClickPreview(
      CameraPushPagePreviewState.startPreview));
}

Future<void> _clickPreview(Action action, Context<CameraPushState> ctx) async {
  CameraPushPagePreviewState previewState = ctx.state.previewState;
  if (previewState == CameraPushPagePreviewState.startPreview) {
    previewState = CameraPushPagePreviewState.stopPreview;
    _alivcLivePusher.stopPreview();
  } else {
    previewState = CameraPushPagePreviewState.startPreview;
    _alivcLivePusher.startPreview();
  }
  ctx.dispatch(CameraPushActionCreator.onClickPreview(previewState));
}

Future<void> _clickPushAction(
    Action action, Context<CameraPushState> ctx) async {
  String pushURL = action.payload ?? "";
  CameraPushPagePushState pushState = ctx.state.pushState;
  if (pushState != CameraPushPagePushState.stopPush) {
    pushState = CameraPushPagePushState.stopPush;
    _alivcLivePusher.stopPush();
  } else {
    pushState = CameraPushPagePushState.startPush;
    _alivcLivePusher.startPushWithURL(pushURL);
  }
  ctx.dispatch(CameraPushActionCreator.onClickPush(pushState));
}

Future<void> _clickPausPush(Action action, Context<CameraPushState> ctx) async {
  CameraPushPagePushState pushState = ctx.state.pushState;
  if (pushState != CameraPushPagePushState.pausePush) {
    pushState = CameraPushPagePushState.pausePush;
    _alivcLivePusher.pause();
  } else {
    pushState = CameraPushPagePushState.startPush;
    _alivcLivePusher.resume();
  }
  ctx.dispatch(CameraPushActionCreator.onClickPausePush(pushState));
}

Future<void> _clickReStartPush(
    Action action, Context<CameraPushState> ctx) async {
  _alivcLivePusher.restartPush();
}

Future<void> _dispose(Action action, Context<CameraPushState> ctx) async {
  _destoryPusher(action, ctx);
}

Future<void> _destoryPusher(Action action, Context<CameraPushState> ctx) async {
  _alivcLivePusher.stopPush();
  _alivcLivePusher.stopPreview();
  _alivcLivePusher.destroy();

  if (ctx.state.beautyOn) {
    _beautyManager.destroyBeauty();
  }
}

Future<void> _clickFlashLamp(
    Action action, Context<CameraPushState> ctx) async {
  bool flashLamp = action.payload ?? false;
  _alivcLivePusher.setFlash(flashLamp);
  ctx.dispatch(CameraPushActionCreator.onClickFlashLamp(flashLamp));
}

Future<void> _showBeauty(Action action, Context<CameraPushState> ctx) async {
  if (ctx.state.beautyOn) {
    _beautyManager.showPanel();
    ctx.dispatch(CameraPushActionCreator.onChangeBeautyPanel(true));
  }
}

Future<void> _hideBeauty(Action action, Context<CameraPushState> ctx) async {
  if (ctx.state.beautyOn) {
    _beautyManager.hidePanel();
    ctx.dispatch(CameraPushActionCreator.onChangeBeautyPanel(false));
  }
}

Future<void> _clickSwitchCamera(
    Action action, Context<CameraPushState> ctx) async {
  _alivcLivePusher.switchCamera();
}

Future<void> _clickSnapShot(Action action, Context<CameraPushState> ctx) async {
  if (Platform.isIOS) {
    _alivcLivePusher.snapshot(1, 0, "snapshot",
        dirTypeForIOS: AlivcLiveSnapshotDirType.document);
  } else {
    CommomUtils.getSystemPath(DirType.externalFile).then((value) {
      _alivcLivePusher.snapshot(1, 0, value);
    });
  }
  _alivcLivePusher.setSnapshotDelegate();
}

//------------------ BGM -----------------
Future<void> _changeEars(Action action, Context<CameraPushState> ctx) async {
  bool enableEars = action.payload ?? false;
  _alivcLivePusher.setBGMEarsBack(enableEars);
  ctx.dispatch(CameraPushActionCreator.onChangeEars(enableEars));
}

Future<void> _changeAudioDenoise(
    Action action, Context<CameraPushState> ctx) async {
  bool enableAudioDenoise = action.payload ?? false;
  _alivcLivePusher.setAudioDenoise(enableAudioDenoise);
  ctx.dispatch(
      CameraPushActionCreator.onChangeAudioDenoise(enableAudioDenoise));
}

Future<void> _changeMute(Action action, Context<CameraPushState> ctx) async {
  bool enableMute = action.payload ?? false;
  _alivcLivePusher.setMute(enableMute);
  ctx.dispatch(CameraPushActionCreator.onChangeMute(enableMute));
}

Future<void> _changePlayState(
    Action action, Context<CameraPushState> ctx) async {
  bool isPlaying = action.payload ?? false;
  if (isPlaying) {
    _alivcLivePusher.pauseBGM();
  } else {
    _alivcLivePusher.resumeBGM();
  }
  ctx.dispatch(CameraPushActionCreator.onChangePlayState(isPlaying));
}

Future<void> _changeLoop(Action action, Context<CameraPushState> ctx) async {
  bool enableLoop = action.payload ?? false;
  _alivcLivePusher.setBGMLoop(enableLoop);
  ctx.dispatch(CameraPushActionCreator.onChangeLoop(enableLoop));
}

Future<void> _changeBGMVolume(
    Action action, Context<CameraPushState> ctx) async {
  int bgmVolume = action.payload ?? 50;
  _alivcLivePusher.setBGMVolume(bgmVolume);
  ctx.dispatch(CameraPushActionCreator.onChangeBGMVolume(bgmVolume));
}

Future<void> _changeCaptureVolume(
    Action action, Context<CameraPushState> ctx) async {
  int captureVolume = action.payload ?? 50;
  _alivcLivePusher.setCaptureVolume(captureVolume);
  ctx.dispatch(CameraPushActionCreator.onChangeCaptureVolume(captureVolume));
}

Future<void> _changeBGMSource(
    Action action, Context<CameraPushState> ctx) async {
  MusicInfo musicInfo = action.payload;

  String path = musicInfo.path;
  if (path.isEmpty) {
    _alivcLivePusher.stopBGMAsync();
  } else if (path.startsWith("http://") || path.startsWith("https://")) {
    _alivcLivePusher.startBGMWithMusicPathAsync(musicInfo.path);
  } else {
    DirType dirType;
    if (Platform.isAndroid) {
      dirType = DirType.externalFile;
    } else {
      dirType = DirType.document;
    }
    var split = musicInfo.path.split("/");
    CommomUtils.getSaveDir(dirType, split[2]).then((value) {
      musicInfo.path = value.path;
      _alivcLivePusher.startBGMWithMusicPathAsync(musicInfo.path);
    });
  }
  ctx.dispatch(CameraPushActionCreator.onChangeBGMSource(musicInfo));
}

Future<void> _goBackPage(Action action, Context<CameraPushState> ctx) async {
  _destoryPusher(action, ctx);
  _resetPageOrientation(action, ctx);
  Navigator.of(ctx.context).pop();
}

Future<void> _setPageOrientation(
    Action action, Context<CameraPushState> ctx) async {
  AlivcLivePushOrientation currentOrientation =
      await ctx.state.pusherConfig.getOrientation();
  if (currentOrientation == AlivcLivePushOrientation.landscape_home_left) {
    SystemChrome.setPreferredOrientations([
      DeviceOrientation.landscapeLeft,
    ]);
  } else if (currentOrientation ==
      AlivcLivePushOrientation.landscape_home_right) {
    SystemChrome.setPreferredOrientations([
      DeviceOrientation.landscapeRight,
    ]);
  }
}

Future<void> _resetPageOrientation(
    Action action, Context<CameraPushState> ctx) async {
  AlivcLivePushOrientation currentOrientation =
      await ctx.state.pusherConfig.getOrientation();
  if (currentOrientation != AlivcLivePushOrientation.portrait) {
    SystemChrome.setPreferredOrientations([
      DeviceOrientation.portraitUp,
    ]);
  }
}

//------------------ Config -----------------
Future<void> _changePushMirror(
    Action action, Context<CameraPushState> ctx) async {
  bool isMirror = action.payload ?? false;
  _alivcLivePusher.setPushMirror(isMirror);
  ctx.dispatch(CameraPushActionCreator.onChangePushMirror(isMirror));
}

Future<void> _changePreviewMirror(
    Action action, Context<CameraPushState> ctx) async {
  bool isMirror = action.payload ?? false;
  _alivcLivePusher.setPreviewMirror(isMirror);
  ctx.dispatch(CameraPushActionCreator.onChangePreviewMirror(isMirror));
}

Future<void> _setPreviewDisplayMode(
    Action action, Context<CameraPushState> ctx) async {
  AlivcPusherPreviewDisplayMode previewMode =
      AlivcPusherPreviewDisplayMode.values[action.payload ?? 1];
  _alivcLivePusher.setPreviewDisplayMode(previewMode);
  ctx.dispatch(
      CameraPushActionCreator.onSetPreviewDisplayMode(previewMode.index));
}

Future<void> _setTargetVideoBitrate(
    Action action, Context<CameraPushState> ctx) async {
  int videoBitrate = action.payload ?? 1400;
  _alivcLivePusher.setTargetVideoBitrate(videoBitrate);
  ctx.dispatch(CameraPushActionCreator.onSetTargetVideoBitrate(videoBitrate));
}

Future<void> _setMinVideoBitrate(
    Action action, Context<CameraPushState> ctx) async {
  int videoBitrate = action.payload ?? 600;
  _alivcLivePusher.setMinVideoBitrate(videoBitrate);
  ctx.dispatch(CameraPushActionCreator.onSetMinVideoBitrate(videoBitrate));
}

//------------------ bottom sheet -----------------
Future<void> _showMusic(Action action, Context<CameraPushState> ctx) async {
  ctx.dispatch(CameraPushActionCreator.onShowMusic());
}

Future<void> _hideMusic(Action action, Context<CameraPushState> ctx) async {
  ctx.dispatch(CameraPushActionCreator.onHideMusic());
}

Future<void> _showMoreSetting(
    Action action, Context<CameraPushState> ctx) async {
  ctx.dispatch(CameraPushActionCreator.onShowMoreSetting());
}

Future<void> _hideMoreSetting(
    Action action, Context<CameraPushState> ctx) async {
  ctx.dispatch(CameraPushActionCreator.onHideMoreSetting());
}
