import 'package:fish_redux/fish_redux.dart';
import 'package:flutter_livepush_demo/page/camera_push/camera_push_action.dart';
import 'package:flutter_livepush_demo/page/camera_push/camera_push_state.dart';

Reducer<CameraPushState> buildReducer() {
  return asReducer(
    <Object, Reducer<CameraPushState>>{
      CameraPushAction.onChangeBeautyPanel: _onChangeBeautyPanel,
      CameraPushAction.onClickFlashLamp: _onClickFlashLamp,
      CameraPushAction.onClickPreview: _onClickPreview,
      CameraPushAction.onClickPush: _onClickPush,
      CameraPushAction.onClickPausePush: _onClickPausePush,
      //BGM
      CameraPushBGMAction.onChangeEars: _onChangeEars,
      CameraPushBGMAction.onChangeAudioDenoise: _onChangeAudioDenoise,
      CameraPushBGMAction.onChangeMute: _onChangeMute,
      CameraPushBGMAction.onChangePlayState: _onChangePlayState,
      CameraPushBGMAction.onChangeLoop: _onChangeLoop,
      CameraPushBGMAction.onChangeBGMVolume: _onChangeBGMVolume,
      CameraPushBGMAction.onChangeCaptureVolume: _onChangeCaptureVolume,
      CameraPushBGMAction.onChangeBGMSource: _onChangeBGMSource,
      CameraPushBGMAction.onUpdateBGMProgress: _onUpdateBGMProgress,
      CameraPushBGMAction.onUpdateBGMDuration: _onUpdateBGMDuration,
      CameraPushAction.updatePushStatusTip: _updatePushStatusTip,
      //config
      CameraPushConfigAction.onChangePushMirror: _onChangePushMirror,
      CameraPushConfigAction.onChangePreviewMirror: _onChangePreviewMirror,
      CameraPushConfigAction.onSetPreviewDisplayMode: _onSetPreviewDisplayMode,
      CameraPushConfigAction.onSetTargetVideoBitrate: _onSetTargetVideoBitrate,
      CameraPushConfigAction.onSetMinVideoBitrate: _onSetMinVideoBitrate,
      //bottom sheet
      CameraPushBottomSheetAction.onShowMusic: _onShowMusic,
      CameraPushBottomSheetAction.onHideMusic: _onHideMusic,
      CameraPushBottomSheetAction.onShowMoreSetting: _onShowMoreSetting,
      CameraPushBottomSheetAction.onHideMoreSetting: _onHideMoreSetting,
    },
  );
}

CameraPushState _onChangeBeautyPanel(CameraPushState state, Action action) {
  final CameraPushState newState = state.clone();
  newState.showBeautyMenu = action.payload ?? false;
  return newState;
}


CameraPushState _onClickFlashLamp(CameraPushState state, Action action) {
  final CameraPushState newState = state.clone();
  newState.isOpenFlashLamp = action.payload ?? false;
  return newState;
}

CameraPushState _onClickPreview(CameraPushState state, Action action) {
  final CameraPushState newState = state.clone();
  newState.previewState =
      action.payload ?? CameraPushPagePreviewState.stopPreview;
  return newState;
}

CameraPushState _onClickPush(CameraPushState state, Action action) {
  final CameraPushState newState = state.clone();
  newState.pushState = action.payload ?? CameraPushPagePushState.stopPush;
  return newState;
}

CameraPushState _onClickPausePush(CameraPushState state, Action action) {
  final CameraPushState newState = state.clone();
  newState.pushState = action.payload ?? CameraPushPagePushState.stopPush;
  return newState;
}

//BGM
CameraPushState _onChangeEars(CameraPushState state, Action action) {
  final CameraPushState newState = state.clone();
  newState.bgmState.isOpenBGMEarsBack = action.payload;
  return newState;
}

CameraPushState _onChangeAudioDenoise(CameraPushState state, Action action) {
  final CameraPushState newState = state.clone();
  newState.bgmState.isOpenAudioDenoise = action.payload;
  return newState;
}

CameraPushState _onChangeMute(CameraPushState state, Action action) {
  final CameraPushState newState = state.clone();
  newState.bgmState.isBGMMute = action.payload;
  return newState;
}

CameraPushState _onChangePlayState(CameraPushState state, Action action) {
  final CameraPushState newState = state.clone();
  newState.bgmState.isBGMPlaying = action.payload;
  return newState;
}

CameraPushState _onChangeLoop(CameraPushState state, Action action) {
  final CameraPushState newState = state.clone();
  newState.bgmState.isBGMLoop = action.payload;
  return newState;
}

CameraPushState _onChangeBGMVolume(CameraPushState state, Action action) {
  final CameraPushState newState = state.clone();
  newState.bgmState.bgmVolume = action.payload;
  return newState;
}

CameraPushState _onChangeCaptureVolume(CameraPushState state, Action action) {
  final CameraPushState newState = state.clone();
  newState.bgmState.bgmCaptureVolume = action.payload;
  return newState;
}

CameraPushState _onChangeBGMSource(CameraPushState state, Action action) {
  final CameraPushState newState = state.clone();
  newState.bgmState.bgmSelectedIndex = action.payload.index;
  return newState;
}

CameraPushState _onUpdateBGMProgress(CameraPushState state, Action action) {
  final CameraPushState newState = state.clone();
  newState.bgmState.bgmProgress = action.payload ?? 0;
  return newState;
}

CameraPushState _onUpdateBGMDuration(CameraPushState state, Action action) {
  final CameraPushState newState = state.clone();
  newState.bgmState.bgmDuration = action.payload ?? 0;
  return newState;
}

CameraPushState _updatePushStatusTip(CameraPushState state, Action action) {
  final CameraPushState newState = state.clone();
  newState.pushStatusTip = action.payload;
  return newState;
}

//------------------ push config -----------------
CameraPushState _onChangePushMirror(CameraPushState state, Action action) {
  bool isMirror = action.payload ?? false;
  final CameraPushState newState = state.clone();
  newState.pushSettingState.isOpenPushMirror = isMirror;
  return newState;
}

CameraPushState _onChangePreviewMirror(CameraPushState state, Action action) {
  bool isMirror = action.payload ?? false;
  final CameraPushState newState = state.clone();
  newState.pushSettingState.isOpenPreviewMirror = isMirror;
  return newState;
}

CameraPushState _onSetPreviewDisplayMode(CameraPushState state, Action action) {
  int modeIndex = action.payload ?? 0;
  final CameraPushState newState = state.clone();
  newState.pushSettingState.previewModelIndex = modeIndex;
  return newState;
}

CameraPushState _onSetTargetVideoBitrate(CameraPushState state, Action action) {
  int videoBitrate = action.payload ?? 1400;
  final CameraPushState newState = state.clone();
  newState.pushSettingState.videoTargetBitrate = videoBitrate;
  return newState;
}

CameraPushState _onSetMinVideoBitrate(CameraPushState state, Action action) {
  int videoBitrate = action.payload ?? 600;
  final CameraPushState newState = state.clone();
  newState.pushSettingState.videoMinBitrate = videoBitrate;
  return newState;
}

//------------------ bottom sheet -----------------
CameraPushState _onShowMusic(CameraPushState state, Action action) {
  final CameraPushState newState = state.clone();
  newState.bottomSheetState = CameraPushPageBottomSheetState.showMusic;
  return newState;
}

CameraPushState _onHideMusic(CameraPushState state, Action action) {
  final CameraPushState newState = state.clone();
  newState.bottomSheetState = CameraPushPageBottomSheetState.nothing;
  return newState;
}

CameraPushState _onShowMoreSetting(CameraPushState state, Action action) {
  final CameraPushState newState = state.clone();
  newState.bottomSheetState = CameraPushPageBottomSheetState.showMorSetting;
  return newState;
}

CameraPushState _onHideMoreSetting(CameraPushState state, Action action) {
  final CameraPushState newState = state.clone();
  newState.bottomSheetState = CameraPushPageBottomSheetState.nothing;
  return newState;
}
