// ignore_for_file: unrelated_type_equality_checks

import 'package:fish_redux/fish_redux.dart';
import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:flutter_livepush_demo/page/camera_push/camera_push_state.dart';
import 'package:flutter_livepush_demo/view/icon_text.dart';
import 'package:flutter_livepush_demo/widgets/camera_push_setting_widget.dart';
import 'package:flutter_livepush_demo/widgets/camera_push_music_widget.dart';
import 'package:flutter_livepush_plugin/beauty/live_push_queen_view.dart';
import 'package:flutter_livepush_plugin/live_pusher_preview.dart';
import 'package:flutter_livepush_demo/page/camera_push/camera_push_action.dart';
import 'package:flutter_gen/gen_l10n/app_localizations.dart';
import 'dart:io';

Widget buildView(
    CameraPushState state, Dispatch dispatch, ViewService viewService) {
  return OrientationBuilder(
    builder: (context, orientation) {
      return Scaffold(
        body: WillPopScope(
          child: Stack(
            fit: StackFit.expand,
            children: [
              _buildPreviewWidget(state, dispatch, viewService),
              _buildBottomWidget(state, viewService, dispatch),
              _buildRightWidget(state, viewService, dispatch),
              _buildTopViewWidget(state, viewService, dispatch),
              _buildQueenWidget(state, viewService, dispatch),
            ],
          ),
          onWillPop: () async {
            return false;
          },
        ),
        bottomSheet: _buildBottomSheet(state, viewService, dispatch),
      );
    },
  );
}

Widget _buildTopViewWidget(
    CameraPushState state, ViewService viewService, Dispatch dispatch) {
  var x = 0.0;
  var y = MediaQuery.of(viewService.context).padding.top;
  var width = MediaQuery.of(viewService.context).size.width;
  var height = 44.0;
  return Positioned(
    left: x,
    top: y,
    width: width,
    height: height,
    child: Container(
        padding: const EdgeInsets.only(left: 8, right: 8),
        width: width,
        height: height,
        child: Row(
          children: [
            IconButton(
              icon: Image.asset(
                "assets/images/ic_back.png",
                width: 32,
                height: 32,
              ),
              onPressed: () {
                dispatch(
                  CameraPushActionCreator.goBackPage(),
                );
              },
            ),
            const SizedBox(width: 8),
            Container(
              width: width - 64 * 2,
              child: Text(
                state.pushStatusTip,
                textAlign: TextAlign.center,
                style: TextStyle(color: Colors.white, fontSize: 16),
              ),
            ),
          ],
        )),
  );
}

Widget _buildPreviewWidget(
    CameraPushState state, Dispatch dispatch, ViewService viewService) {
  var x = 0.0;
  var y = 0.0;
  var width = MediaQuery.of(viewService.context).size.width;
  var height = MediaQuery.of(viewService.context).size.height;

  AlivcPusherPreviewType viewType;
  if (Platform.isAndroid) {
    viewType = AlivcPusherPreviewType.base;
  } else {
    viewType = AlivcPusherPreviewType.push;
  }

  AlivcPusherPreview pusherPreviewView = AlivcPusherPreview(
    viewType: viewType,
    onCreated: (id) async {
      await Future.delayed(const Duration(milliseconds: 500));
      dispatch(CameraPushActionCreator.previewCreated());
    },
    x: x,
    y: y,
    width: width,
    height: height,
  );
  return Positioned(
    child: Container(
        color: Colors.black,
        width: width,
        height: height,
        child: pusherPreviewView),
  );
}

Widget _buildRightWidget(
    CameraPushState state, ViewService viewService, Dispatch dispatch) {
  var right = MediaQuery.of(viewService.context).padding.right;
  var height = MediaQuery.of(viewService.context).size.height;
  var childrenItemCount = 4;

  List<Widget> childrenView = [
    IconText(
      text: AppLocalizations.of(viewService.context)!.camerapush_music,
      onIconClick: (isSelected) {
        dispatch(
          CameraPushActionCreator.showMusic(),
        );
      },
      normalIcon: "assets/images/ic_music.png",
    ),
  ];
  if (state.beautyOn) {
    childrenView.addAll([
      const SizedBox(height: 20.0),
      IconText(
        text: AppLocalizations.of(viewService.context)!.camerapush_beauty,
        onIconClick: (isSelected) {
          if (state.showBeautyMenu) {
            dispatch(
              CameraPushActionCreator.hideBeauty(),
            );
          } else {
            dispatch(
              CameraPushActionCreator.showBeauty(),
            );
          }
        },
        isSelected: state.showBeautyMenu,
        normalIcon: "assets/images/ic_beauty.png",
      ),
    ]);
    childrenItemCount += 1;
  }
  childrenView.addAll([
    const SizedBox(height: 20.0),
    IconText(
      text: AppLocalizations.of(viewService.context)!.camerapush_flash,
      onIconClick: (isSelected) {
        dispatch(
          CameraPushActionCreator.clickFlashLamp(!isSelected),
        );
      },
      isSelected: state.isOpenFlashLamp,
      normalIcon: "assets/images/ic_flash_off.png",
      selectedIcon: "assets/images/ic_flash_on.png",
    ),
    const SizedBox(height: 20.0),
    IconText(
      text: AppLocalizations.of(viewService.context)!.camerapush_camera,
      onIconClick: (isSelected) {
        dispatch(CameraPushActionCreator.clickSwitchCamera());
      },
      normalIcon: "assets/images/ic_camera_on.png",
    ),
    const SizedBox(height: 20.0),
    IconText(
      text: AppLocalizations.of(viewService.context)!.camerapush_snapshot,
      onIconClick: (isSelected) {
        dispatch(CameraPushActionCreator.clickSnapShot());
      },
      normalIcon: "assets/images/ic_screenshot.png",
    ),
  ]);

  return Positioned(
    right: right == 0 ? 20.0 : right,
    top: (height - 20.0 * (childrenItemCount - 1) - 45 * childrenItemCount) /
        2.0,
    child: Column(
      children: childrenView,
    ),
  );
}

Widget _buildBottomWidget(
    CameraPushState state, ViewService viewService, Dispatch dispatch) {
  return Positioned(
      bottom: 0.0,
      left: 0.0,
      right: 0.0,
      child: Container(
        color: const Color(0x66FCFCFD),
        child: SingleChildScrollView(
            scrollDirection: Axis.horizontal,
            padding: const EdgeInsets.only(
                left: 20.0, bottom: 10.0, top: 10.0, right: 20.0),
            child: Row(
              children: [
                const SizedBox(width: 10.0),
                IconText(
                  text: AppLocalizations.of(viewService.context)!
                      .camerapush_start_preview,
                  onIconClick: (isSelected) {
                    dispatch(CameraPushActionCreator.clickPreview());
                  },
                  isSelected: state.previewState ==
                          CameraPushPagePreviewState.startPreview &&
                      state.pushState != CameraPushPagePushState.pausePush,
                  normalIcon: "assets/images/ic_preview_off.png",
                  selectedIcon: "assets/images/ic_preview_on.png",
                  selectedTextColor: const Color(0xFF4DCFE1),
                ),
                const SizedBox(width: 20.0),
                IconText(
                  text: AppLocalizations.of(viewService.context)!
                      .camerapush_start_pushing,
                  onIconClick: (isSelected) {
                    dispatch(CameraPushActionCreator.clickPush(state.pushURL));
                  },
                  isSelected:
                      state.pushState != CameraPushPagePushState.stopPush,
                  normalIcon: "assets/images/ic_push_off.png",
                  selectedIcon: "assets/images/ic_push_on.png",
                  selectedTextColor: const Color(0xFF4DCFE1),
                ),
                const SizedBox(width: 20.0),
                IconText(
                  text: AppLocalizations.of(viewService.context)!
                      .camerapush_pause,
                  onIconClick: (isSelected) {
                    dispatch(CameraPushActionCreator.clickPausPush());
                  },
                  isSelected:
                      state.pushState == CameraPushPagePushState.pausePush,
                  normalIcon: "assets/images/ic_push_pause.png",
                  selectedTextColor: const Color(0xFF4DCFE1),
                ),
                const SizedBox(width: 20.0),
                IconText(
                  text: AppLocalizations.of(viewService.context)!
                      .camerapush_restart_pushing,
                  onIconClick: (isSelected) {
                    dispatch(CameraPushActionCreator.clickReStartPush());
                  },
                  normalIcon: "assets/images/ic_repush_off.png",
                ),
                const SizedBox(width: 20.0),
                IconText(
                  text: AppLocalizations.of(viewService.context)!
                      .camerapush_more_setting,
                  onIconClick: (isSelected) {
                    dispatch(
                      CameraPushActionCreator.showMoreSetting(),
                    );
                  },
                  normalIcon: "assets/images/ic_setting_more_on.png",
                ),
                const SizedBox(width: 20.0),
                IconText(
                  text: AppLocalizations.of(viewService.context)!
                      .camerapush_more_data,
                  onIconClick: (isSelected) {},
                  normalIcon: "assets/images/ic_push_info_on.png",
                )
              ],
            )),
      ));
}

Widget _buildBottomSheet(
    CameraPushState state, ViewService viewService, Dispatch dispatch) {
  switch (state.bottomSheetState) {
    case CameraPushPageBottomSheetState.showMorSetting:
      return showSetting(viewService.context, state, dispatch);
    case CameraPushPageBottomSheetState.showMusic:
      return bottomSheetMusic(state, viewService, dispatch);
    case CameraPushPageBottomSheetState.nothing:
      return const SizedBox();
  }
}

Widget _buildQueenWidget(
    CameraPushState state, ViewService viewService, Dispatch dispatch) {
  if (state.showBeautyMenu && Platform.isAndroid) {
    var x = 0.0;
    var y = 0.0;
    var width = MediaQuery.of(viewService.context).size.width;
    var height = MediaQuery.of(viewService.context).size.height / 3;
    return Positioned(
      left: 0,
      right: 0,
      bottom: 0,
      child: SizedBox(
        width: width,
        height: height,
        child: LivePushQueenView(
          x: x,
          y: y,
          width: width,
          height: height,
          onCreated: (id) {},
        ),
      ),
    );
  } else {
    return const SizedBox();
  }
}
