import 'package:fish_redux/fish_redux.dart';
import 'package:flutter/material.dart';
import 'package:flutter/widgets.dart';
import 'package:flutter_livepush_demo/page/home/home_action.dart';
import 'package:flutter_livepush_demo/page/home/home_state.dart';
import 'package:flutter_gen/gen_l10n/app_localizations.dart';

Widget buildView(
  HomeState state,
  Dispatch dispatch,
  ViewService viewService,
) {
  return Scaffold(
    body: Column(
      children: [
        Stack(
          alignment: Alignment.bottomLeft,
          children: [
            Image.asset("assets/images/homepage_bg.jpg"),
            Container(
              padding: const EdgeInsets.all(22.0),
              child: Text(
                AppLocalizations.of(viewService.context)!.home_theme,
                style: const TextStyle(fontSize: 42.0),
              ),
            )
          ],
        ),
        Expanded(
          child: GridView.count(
            crossAxisCount: 2,
            mainAxisSpacing: 0,
            crossAxisSpacing: 0,
            childAspectRatio: 1 / 1,
            children: state.tabs
                .map((e) => _buildItem(e, dispatch, viewService))
                .toList(),
          ),
        ),
        GestureDetector(
          onTap: () {
            dispatch(
              HomeActionCreator.clickAliyunPrivacyPolicy(),
            );
          },
          child: Text(
            AppLocalizations.of(viewService.context)!.home_privacy_policy,
            style: const TextStyle(color: Color(0xff888888), fontSize: 12),
          ),
        ),
        const SizedBox(height: 5),
        Text(
          AppLocalizations.of(viewService.context)!.home_bottom_tip,
          style: const TextStyle(color: Color(0xff888888), fontSize: 11),
        ),
        const SizedBox(height: 5),
        Text(
          AppLocalizations.of(viewService.context)!.home_version +
              state.sdkVersion,
          style: const TextStyle(color: Color(0xff888888), fontSize: 11),
        )
      ],
    ),
  );
}

Widget _buildItem(HomeTab e, Dispatch dispatch, ViewService viewService) =>
    GestureDetector(
      onTap: () async {
        switch (e.modelType) {
          case ModelType.cameraPush:
            dispatch(HomeActionCreator.clickCameraPushAction());
            break;
          case ModelType.interactiveLive:
            dispatch(HomeActionCreator.clickInteractiveLiveAction());
            break;
          case ModelType.interactivePK:
            dispatch(HomeActionCreator.clickInteractivePKAction());
            break;
        }
      },
      child: Container(
        margin: const EdgeInsets.all(15.0),
        padding: const EdgeInsets.only(left: 10.0),
        alignment: Alignment.centerLeft,
        decoration: BoxDecoration(
          //边框
          border: Border.all(color: const Color(0xFFEDEDED), width: 2.0),
          //圆角
          borderRadius: BorderRadius.circular(8.0),
        ),
        child: Column(
          mainAxisAlignment: MainAxisAlignment.spaceAround,
          children: [
            Image.asset(e.iconPath, width: 48.0, height: 48.0),
            Text(
              AppLocalizations.of(viewService.context)!.home_page_name(e.title),
              style: const TextStyle(fontSize: 17.0),
            )
          ],
        ),
      ),
    );
