import 'package:fish_redux/fish_redux.dart';
import 'package:flutter/cupertino.dart' hide Action;
import 'package:flutter_livepush_demo/common/CommonUtils.dart';
import 'package:flutter_livepush_demo/common/InteractiveUrlUtils.dart';
import 'package:flutter_livepush_demo/manager/push_config_manager.dart';
import 'package:flutter_livepush_demo/page/interactive/interactive_info/interactive_info_page.dart';
import 'package:flutter_livepush_demo/page/interactive/interactive_mode/interactive_mode_page.dart';
import 'package:flutter_livepush_demo/page/push_config/push_config_page.dart';
import 'package:flutter_livepush_plugin/live_push_config.dart';
import 'package:flutter_livepush_plugin/live_pusher.dart';

import 'interactive_enrollment_action.dart';
import 'interactive_enrollment_state.dart';

Effect<InteractiveEnrollmentState> buildEffect() {
  return combineEffects(<Object, Effect<InteractiveEnrollmentState>>{
    Lifecycle.initState: _init,
    Lifecycle.dispose: _dispose,
    InteractiveEnrollmentAction.clickNext: _clickNext,
    InteractiveEnrollmentAction.clickGoBack: _clickGoBack,
    InteractiveEnrollmentAction.settingParameters: _settingParameters,
    InteractiveEnrollmentAction.editAppInfo: _editAppInfo,
    InteractiveEnrollmentAction.changeIdentity: _changeIdentity,
    InteractiveEnrollmentAction.changeAppInfoState: _changeAppInfoState,
    InteractiveEnrollmentAction.changeUserId: _changeUserId,
    InteractiveEnrollmentAction.changeRoomId: _changeRoomId,
  });
}

late AlivcLivePusher _alivcLivePusher;
late AlivcLivePusherConfig _alivcLivePusherConfig;

Future<void> _init(
    Action action, Context<InteractiveEnrollmentState> ctx) async {
  ConfigData.resetData();
  _alivcLivePusher = AlivcLivePusher.init();
  _alivcLivePusher.createConfig();
  _alivcLivePusherConfig = AlivcLivePusherConfig.init();
}

Future<void> _dispose(
    Action action, Context<InteractiveEnrollmentState> ctx) async {}

Future<void> _clickNext(
    Action action, Context<InteractiveEnrollmentState> ctx) async {
  InteractiveEnrollmentState state = ctx.state;

  ///设置 appInfo，用于生成
  InteractiveUrlUtils.setAppInfo(state.appId, state.appKey, state.playDomain);

  Navigator.of(ctx.context).push(
    CupertinoPageRoute(
      builder: (_) => InteractiveModePage().buildPage(
        {
          interactiveTypeKey: state.interactiveTypeEnum,
          interactiveIdentityKey: state.interactiveIdentityTypeEnum,
          interactiveAppIdKey: state.appId,
          interactiveAppKeyKey: state.appKey,
          interactivePlayDomainKey: state.playDomain,
          interactiveRoomIdKey: state.roomId,
          interactiveUserIdKey: state.userId,
          pushConfigKey: _alivcLivePusherConfig,
        },
      ),
    ),
  );
}

Future<void> _clickGoBack(
    Action action, Context<InteractiveEnrollmentState> ctx) async {
  Navigator.of(ctx.context).pop();
}

Future<void> _settingParameters(
    Action action, Context<InteractiveEnrollmentState> ctx) async {
  Navigator.of(ctx.context).push(
    CupertinoPageRoute(
        builder: (_) => ConfigPage().buildPage({
              pushConfigPageTypeKey: PushConfigPageType.interactive,
              pushConfigOnSettingConfigActionKey:
                  (AlivcLivePusherConfig pusherConfig) {
                _alivcLivePusherConfig = pusherConfig;
              },
            })),
  );
}

Future<void> _editAppInfo(
    Action action, Context<InteractiveEnrollmentState> ctx) async {
  InteractiveEnrollmentState state = ctx.state;
  Navigator.of(ctx.context)
      .push(
        CupertinoPageRoute(
          builder: (_) => InteractiveInfoPage().buildPage(
            {
              interactiveTypeKey: state.interactiveTypeEnum,
              interactiveAppIdKey: state.appId,
              interactiveAppKeyKey: state.appKey,
              interactivePlayDomainKey: state.playDomain,
              interactiveFromHomePageKey: false
            },
          ),
        ),
      )
      .then(
        (value) => {
          if (value != null)
            {
              ctx.dispatch(
                  InteractiveEnrollmentActionCreator.onEditAppInfo(value))
            }
        },
      );
}

Future<void> _changeIdentity(
    Action action, Context<InteractiveEnrollmentState> ctx) async {
  ctx.dispatch(
      InteractiveEnrollmentActionCreator.onChangeIdentity(action.payload));
}

Future<void> _changeAppInfoState(
    Action action, Context<InteractiveEnrollmentState> ctx) async {
  ctx.dispatch(
      InteractiveEnrollmentActionCreator.onChangeAppInfoState(action.payload));
}

Future<void> _changeUserId(
    Action action, Context<InteractiveEnrollmentState> ctx) async {
  ctx.dispatch(
      InteractiveEnrollmentActionCreator.onChangeUserId(action.payload));
}

Future<void> _changeRoomId(
    Action action, Context<InteractiveEnrollmentState> ctx) async {
  ctx.dispatch(
      InteractiveEnrollmentActionCreator.onChangeRoomId(action.payload));
}
