import 'package:fish_redux/fish_redux.dart';
import 'package:flutter/material.dart';
import 'package:flutter_livepush_demo/common/CommonUtils.dart';
import 'package:flutter_livepush_demo/page/interactive/interactive_enrollment/interactive_enrollment_action.dart';
import 'package:flutter_livepush_demo/view/interactive_input_view.dart';
import 'package:flutter_livepush_demo/view/push_button.dart';
import 'package:flutter_livepush_demo/widgets/custom_radio_widget.dart';
import 'package:flutter_livepush_demo/widgets/interactive_input_appbar.dart';
import 'interactive_enrollment_state.dart';
import 'package:flutter_gen/gen_l10n/app_localizations.dart';

Widget buildView(
  InteractiveEnrollmentState state,
  Dispatch dispatch,
  ViewService viewService,
) {
  String title;
  if (state.interactiveTypeEnum == InteractiveTypeEnum.interactiveLive) {
    title = AppLocalizations.of(viewService.context)!.interactive_co_streaming;
  } else {
    title = AppLocalizations.of(viewService.context)!.interactive_battles;
  }
  return Scaffold(
    appBar: buildAppBar(
      title,
      leadingClickListener: () {
        dispatch(InteractiveEnrollmentActionCreator.clickGoBack());
      },
      showAction: true,
      actionTitle: AppLocalizations.of(viewService.context)!.config_page_theme,
      actionIconClickListener: () {
        dispatch(InteractiveEnrollmentActionCreator.settingParameters());
      },
    ),
    body: Container(
      color: Colors.black,
      child: Container(
        margin: const EdgeInsets.only(left: 20, right: 20),
        child: Stack(
          children: [
            _buildInputView(state, dispatch, viewService),
            _buildBottomButton(state, dispatch, viewService),
          ],
        ),
      ),
    ),
  );
}

///input UserId and RoomId
Widget _buildInputView(InteractiveEnrollmentState state, Dispatch dispatch,
    ViewService viewService) {
  return SizedBox.expand(
    child: SingleChildScrollView(
      child: Column(
        children: [
          const SizedBox(height: 40),
          InteractiveInputView(
            title:
                AppLocalizations.of(viewService.context)!.interactive_user_id,
            hint: AppLocalizations.of(viewService.context)!
                .interactive_input_number_digits,
            onlyClearIcon: true,
            action: (text) {
              dispatch(InteractiveEnrollmentActionCreator.changeUserId(text));
            },
          ),
          const SizedBox(height: 20),
          InteractiveInputView(
            title:
                AppLocalizations.of(viewService.context)!.interactive_room_id,
            hint: AppLocalizations.of(viewService.context)!
                .interactive_input_number_digits,
            onlyClearIcon: true,
            action: (text) {
              dispatch(InteractiveEnrollmentActionCreator.changeRoomId(text));
            },
          ),
          const SizedBox(height: 20),
          _buildAppInfoWidget(state, dispatch, viewService),
        ],
      ),
    ),
  );
}

Widget _buildBottomButton(InteractiveEnrollmentState state, Dispatch dispatch,
    ViewService viewService) {
  return Positioned(
    left: 0,
    right: 0,
    bottom: 20,
    child: PushButton(
      enable: state.userId.isNotEmpty && state.roomId.isNotEmpty,
      title: AppLocalizations.of(viewService.context)!.interactive_next_button,
      action: () {
        dispatch(InteractiveEnrollmentActionCreator.clickNext());
      },
    ),
  );
}

Widget _buildAppInfoWidget(InteractiveEnrollmentState state, Dispatch dispatch,
    ViewService viewService) {
  return Column(
    crossAxisAlignment: CrossAxisAlignment.end,
    children: [
      Row(
        mainAxisAlignment: MainAxisAlignment.spaceBetween,
        children: [
          Text(
            AppLocalizations.of(viewService.context)!
                .interactive_application_information,
            style: const TextStyle(color: Colors.white),
          ),
          IconButton(
            alignment: Alignment.centerRight,
            padding: const EdgeInsets.all(0.0),
            onPressed: () {
              dispatch(InteractiveEnrollmentActionCreator.changeAppInfoState(
                  !state.showAppInfo));
            },
            icon: Transform.rotate(
              angle: state.arrowAngle,
              child: Image.asset(
                "assets/images/ic_chevron_down.png",
                color: Colors.white,
                width: 20,
                height: 20,
              ),
            ),
          ),
        ],
      ),
      _showAppInfoView(state, dispatch, viewService),
      const Divider(color: Colors.white),
      const SizedBox(height: 20),
      _buildIdentityView(state, dispatch, viewService),
    ],
  );
}

///app info
Widget _showAppInfoView(InteractiveEnrollmentState state, Dispatch dispatch,
    ViewService viewService) {
  if (state.showAppInfo) {
    return Column(
      crossAxisAlignment: CrossAxisAlignment.end,
      children: [
        _createAppInfoView(
            AppLocalizations.of(viewService.context)!.interactive_app_id,
            state.appId),
        const SizedBox(height: 10),
        _createAppInfoView(
            AppLocalizations.of(viewService.context)!.interactive_app_key,
            state.appKey),
        const SizedBox(height: 10),
        _createAppInfoView(
            AppLocalizations.of(viewService.context)!.interactive_play_domain,
            state.playDomain),
        IconButton(
          alignment: Alignment.centerRight,
          padding: const EdgeInsets.all(0.0),
          onPressed: () {
            dispatch(InteractiveEnrollmentActionCreator.editAppInfo());
          },
          icon: Image.asset(
            "assets/images/ic_edit.png",
            width: 18,
            height: 18,
          ),
        ),
      ],
    );
  } else {
    return const SizedBox();
  }
}

///custom widget
Widget _createAppInfoView(String title, String value) {
  return Row(
    mainAxisAlignment: MainAxisAlignment.spaceBetween,
    children: [
      Text(
        title,
        style: const TextStyle(color: Color(0xFFB2B7C4), fontSize: 12),
      ),
      Text(
        value,
        style: const TextStyle(color: Color(0xFFB2B7C4), fontSize: 12),
      ),
    ],
  );
}

///身份选择 Widget
Widget _buildIdentityView(InteractiveEnrollmentState state, Dispatch dispatch,
    ViewService viewService) {
  if (state.interactiveTypeEnum == InteractiveTypeEnum.interactiveLive) {
    return Row(
      mainAxisAlignment: MainAxisAlignment.spaceAround,
      children: [
        CustomRadioWidget(
          title: AppLocalizations.of(viewService.context)!
              .interactive_co_streaming_streamer,
          value: InteractiveIdentityTypeEnum.anchor,
          groupValue: state.interactiveIdentityTypeEnum,
          onChanged: (e) {
            dispatch(InteractiveEnrollmentActionCreator.changeIdentity(e));
          },
        ),
        CustomRadioWidget(
          title: AppLocalizations.of(viewService.context)!
              .interactive_co_streaming_viewers,
          value: InteractiveIdentityTypeEnum.audience,
          groupValue: state.interactiveIdentityTypeEnum,
          onChanged: (e) {
            dispatch(InteractiveEnrollmentActionCreator.changeIdentity(e));
          },
        ),
      ],
    );
  } else {
    return const SizedBox();
  }
}
