import 'package:fish_redux/fish_redux.dart';
import 'package:flutter/material.dart';
import 'package:flutter_livepush_demo/page/interactive/interactive_info/interactive_info_state.dart';
import 'package:flutter_livepush_demo/view/interactive_input_view.dart';
import 'package:flutter_livepush_demo/view/push_button.dart';
import 'package:flutter_livepush_demo/widgets/interactive_input_appbar.dart';
import 'package:flutter_gen/gen_l10n/app_localizations.dart';
import 'interactive_info_action.dart';

Widget buildView(
  InteractiveInfoState state,
  Dispatch dispatch,
  ViewService viewService,
) {
  return Scaffold(
    appBar: buildAppBar(
      AppLocalizations.of(viewService.context)!
          .interactive_application_information,
      leadingClickListener: () {
        dispatch(InteractiveInfoActionCreator.clickGoBack());
      },
    ),
    body: Container(
      color: Colors.black,
      child: Container(
        margin: const EdgeInsets.only(left: 20, right: 20),
        child: Stack(
          children: [
            _buildInputView(dispatch, state, viewService),
            _buildBottomButton(dispatch, state, viewService),
          ],
        ),
      ),
    ),
  );
}

Widget _buildInputView(
    Dispatch dispatch, InteractiveInfoState state, ViewService viewService) {
  return SizedBox.expand(
    child: SingleChildScrollView(
      child: Column(
        children: [
          const SizedBox(height: 40),
          InteractiveInputView(
            title: AppLocalizations.of(viewService.context)!.interactive_app_id,
            hint: AppLocalizations.of(viewService.context)!
                .interactive_input_number_digits,
            text: state.appId,
            action: (text) {
              dispatch(InteractiveInfoActionCreator.changeAppId(text));
            },
          ),
          const SizedBox(height: 20),
          InteractiveInputView(
            title:
                AppLocalizations.of(viewService.context)!.interactive_app_key,
            hint: AppLocalizations.of(viewService.context)!
                .interactive_input_number_digits,
            text: state.appKey,
            action: (text) {
              dispatch(InteractiveInfoActionCreator.changeAppKey(text));
            },
          ),
          const SizedBox(height: 20),
          InteractiveInputView(
            title: AppLocalizations.of(viewService.context)!
                .interactive_play_domain,
            hint: AppLocalizations.of(viewService.context)!
                .interactive_input_number_digits,
            text: state.playDomain,
            action: (text) {
              dispatch(InteractiveInfoActionCreator.changePlayDomain(text));
            },
          ),
        ],
      ),
    ),
  );
}

Widget _buildBottomButton(
    Dispatch dispatch, InteractiveInfoState state, ViewService viewService) {
  return Positioned(
    left: 0,
    right: 0,
    bottom: 20,
    child: PushButton(
      title: AppLocalizations.of(viewService.context)!.interactive_next_button,
      enable: state.appId.isNotEmpty &&
          state.appKey.isNotEmpty &&
          state.playDomain.isNotEmpty,
      action: () {
        dispatch(InteractiveInfoActionCreator.clickNext());
      },
    ),
  );
}
