import 'package:fish_redux/fish_redux.dart';
import 'package:flutter/material.dart';
import 'package:flutter_aliplayer/flutter_aliplayer.dart';
import 'package:flutter_livepush_demo/common/CommonUtils.dart';
import 'package:flutter_livepush_demo/page/interactive/interactive_mode/interactive_mode_action.dart';
import 'package:flutter_livepush_demo/view/common_view.dart';
import 'package:flutter_livepush_demo/view/push_button.dart';
import 'package:flutter_livepush_demo/widgets/interactive_meeting_info_dialog.dart';
import 'package:flutter_gen/gen_l10n/app_localizations.dart';
import 'package:flutter_livepush_plugin/live_pusher_preview.dart';
import 'package:fluttertoast/fluttertoast.dart';
import 'interactive_mode_state.dart';

Widget buildView(
  InteractiveModeState state,
  Dispatch dispatch,
  ViewService viewService,
) {
  Widget contentWidget;
  if (state.typeEnum == InteractiveTypeEnum.interactiveLive) {
    contentWidget = _buildInteractiveWidget(state, dispatch, viewService);
  } else {
    contentWidget = _buildInteractivePKWidget(state, dispatch, viewService);
  }
  return Scaffold(
    body: WillPopScope(
      child: Container(
        color: Colors.black,
        child: Stack(
          fit: StackFit.expand,
          children: [
            contentWidget,
            _buildTitleWidget(state, dispatch, viewService),
            _buildFunctionWidget(state, dispatch),
          ],
        ),
      ),
      onWillPop: () async {
        return false;
      },
    ),
  );
}

///顶部控件
Widget _buildTitleWidget(
    InteractiveModeState state, Dispatch dispatch, ViewService viewService) {
  return Positioned(
    top: 44,
    left: 10,
    right: 10,
    child: Row(
      mainAxisAlignment: MainAxisAlignment.spaceBetween,
      children: [
        IconButton(
          icon: Image.asset(
            "assets/images/ic_close_white.png",
            width: 28,
            height: 28,
            color: Colors.white,
          ),
          onPressed: () {
            String _goBackTip = AppLocalizations.of(viewService.context)!
                .intercative_room_back_tip;
            if (state.typeEnum == InteractiveTypeEnum.interactiveLive) {
              if (state.identityTypeEnum ==
                  InteractiveIdentityTypeEnum.anchor) {
                if (state.pullStatus == InteractivePullStatus.pulling) {
                  _goBackTip = AppLocalizations.of(viewService.context)!
                      .interactive_co_streaming_close_co_streaming_room_tip;
                }
              } else {
                if (state.pushStatus == InteractivePushStatus.pushing) {
                  _goBackTip = AppLocalizations.of(viewService.context)!
                      .interactive_co_streaming_close_co_streaming_room_tip;
                }
              }
            } else {
              if (state.pullStatus == InteractivePullStatus.pulling) {
                _goBackTip = AppLocalizations.of(viewService.context)!
                    .interactive_battles_close_pk_room_tip;
              }
            }
            showAlert(viewService.context, _goBackTip, onOk: () {
              dispatch(InteractiveModeActionCreator.goBackPage());
            });
          },
        ),
        Text(
          state.roomId,
          style: const TextStyle(fontSize: 16, color: Colors.white),
        ),
        const SizedBox(width: 28),
      ],
    ),
  );
}

///右侧功能控件
Widget _buildFunctionWidget(InteractiveModeState state, Dispatch dispatch) {
  String muteImageAsset;
  if (state.mute) {
    muteImageAsset = "assets/images/ic_volume_off.png";
  } else {
    muteImageAsset = "assets/images/ic_volume_on.png";
  }
  return Positioned(
    top: 44,
    right: 10,
    child: Column(
      children: [
        IconButton(
          onPressed: () {
            dispatch(InteractiveModeActionCreator.clickSwitchCamera());
          },
          icon: Image.asset(
            "assets/images/ic_camera.png",
            width: 22,
            height: 22,
          ),
        ),
        IconButton(
          onPressed: () {
            dispatch(InteractiveModeActionCreator.clickMute(!state.mute));
          },
          icon: Image.asset(
            muteImageAsset,
            width: 22,
            height: 22,
          ),
        ),
        IconButton(
          onPressed: () {
            dispatch(InteractiveModeActionCreator.clickSwitchSpeaker());
          },
          icon: Image.asset(
            "assets/images/ic_speaker_phone.png",
            width: 22,
            height: 22,
          ),
        ),
      ],
    ),
  );
}

///连麦Button
Widget _buildInteractiveButton(InteractiveModeState state, Dispatch dispatch,
    ViewService viewService, String title) {
  double? left;
  double? right;
  double? top;
  double? bottom;
  double width = 94;
  double height = 35;
  if (state.typeEnum == InteractiveTypeEnum.interactiveLive) {
    right = MediaQuery.of(viewService.context).padding.right + 16;
    bottom = MediaQuery.of(viewService.context).padding.bottom + 16;
  } else {
    top = MediaQuery.of(viewService.context).size.height / 2.1 + 16;
    left = (MediaQuery.of(viewService.context).size.width - width) / 2.0;
  }

  Color buttonColor = Colors.white;
  if (state.identityTypeEnum == InteractiveIdentityTypeEnum.anchor) {
    if (state.pullStatus == InteractivePullStatus.pulling ||
        state.pullStatus == InteractivePullStatus.error) {
      buttonColor = Colors.red;
    } else {
      buttonColor = Colors.blue;
    }
  } else {
    if (state.pushStatus == InteractivePushStatus.pushing ||
        state.pushStatus == InteractivePushStatus.error) {
      buttonColor = Colors.red;
    } else {
      buttonColor = Colors.blue;
    }
  }

  return positionedView(
    left: left,
    right: right,
    top: top,
    bottom: bottom,
    width: width,
    height: height,
    child: PushButton(
      width: 94,
      height: 30,
      titleSize: 14,
      color: buttonColor,
      title: title,
      action: () async {
        if (state.typeEnum == InteractiveTypeEnum.interactiveLive) {
          //连麦互动
          _startInteractiveAction(state, dispatch, viewService);
        } else {
          //PK
          _startInteractivePKAction(state, dispatch, viewService);
        }
      },
    ),
  );
}

//互动连麦，点击开始连麦/停止连麦
_startInteractiveAction(
    InteractiveModeState state, Dispatch dispatch, ViewService viewService) {
  if (state.identityTypeEnum == InteractiveIdentityTypeEnum.anchor) {
    //主播身份
    if (state.pullStatus == InteractivePullStatus.pulling ||
        state.pullStatus == InteractivePullStatus.error) {
      showAlert(
        viewService.context,
        AppLocalizations.of(viewService.context)!
            .interactive_co_streaming_stop_link_mic_tip,
        onOk: () {
          dispatch(InteractiveModeActionCreator.stopLink());
        },
      );
    } else {
      _showDialog(
        state,
        dispatch,
        viewService.context,
        pressHandle: () {
          dispatch(InteractiveModeActionCreator.startLink());
        },
      );
    }
  } else {
    //观众身份
    if (state.pushStatus == InteractivePushStatus.pushing ||
        state.pushStatus == InteractivePushStatus.error) {
      showAlert(
        viewService.context,
        AppLocalizations.of(viewService.context)!
            .interactive_co_streaming_stop_link_mic_tip,
        onOk: () {
          dispatch(InteractiveModeActionCreator.stopLink());
        },
      );
    } else {
      dispatch(InteractiveModeActionCreator.startLink());
    }
  }
}

//PK互动，点击开始PK/停止PK
_startInteractivePKAction(
    InteractiveModeState state, Dispatch dispatch, ViewService viewService) {
  if (state.pullStatus == InteractivePullStatus.pulling ||
      state.pullStatus == InteractivePullStatus.error) {
    showAlert(
      viewService.context,
      AppLocalizations.of(viewService.context)!.interactive_battles_stop_pk_tip,
      onOk: () {
        dispatch(InteractiveModeActionCreator.stopLink());
      },
    );
  } else {
    _showDialog(
      state,
      dispatch,
      viewService.context,
      pressHandle: () {
        dispatch(InteractiveModeActionCreator.startLink());
      },
    );
  }
}

//---------------------------------------------------------------------------------------------------------------
///连麦互动
Widget _buildInteractiveWidget(
    InteractiveModeState state, Dispatch dispatch, ViewService viewService) {
  String title = "";
  if (state.identityTypeEnum == InteractiveIdentityTypeEnum.anchor) {
    if (state.pullStatus == InteractivePullStatus.pulling ||
        state.pullStatus == InteractivePullStatus.error) {
      title = AppLocalizations.of(viewService.context)!
          .interactive_co_streaming_end_link_mic;
    } else {
      title = AppLocalizations.of(viewService.context)!
          .interactive_co_streaming_start_link_mic;
    }
  } else {
    if (state.pushStatus == InteractivePushStatus.pushing ||
        state.pushStatus == InteractivePushStatus.error) {
      title = AppLocalizations.of(viewService.context)!
          .interactive_co_streaming_end_link_mic;
    } else {
      title = AppLocalizations.of(viewService.context)!
          .interactive_co_streaming_start_link_mic;
    }
  }

  return Stack(
    children: [
      _buildRenderPreview(state, dispatch, viewService),
      _buildRenderCustomerView(state, dispatch, viewService),
      _buildInteractiveButton(state, dispatch, viewService, title),
    ],
  );
}

//---------------------------------------------------------------------------------------------------------------
///PK互动
Widget _buildInteractivePKWidget(
    InteractiveModeState state, Dispatch dispatch, ViewService viewService) {
  String title = "";
  if (state.pullStatus == InteractivePullStatus.pulling ||
      state.pullStatus == InteractivePullStatus.error) {
    title =
        AppLocalizations.of(viewService.context)!.interactive_battles_end_pk;
  } else {
    title =
        AppLocalizations.of(viewService.context)!.interactive_battles_start_pk;
  }

  return Positioned(
    left: 0.0,
    top: MediaQuery.of(viewService.context).padding.top + 100,
    width: MediaQuery.of(viewService.context).size.width,
    height: MediaQuery.of(viewService.context).size.height / 2.1 + 16 + 35,
    child: Stack(
      children: [
        _buildRenderPreview(state, dispatch, viewService),
        _buildRenderCustomerView(state, dispatch, viewService),
        _buildInteractiveButton(state, dispatch, viewService, title),
        Align(
          alignment: Alignment.topCenter,
          child: Image.asset(
            "assets/images/ic_pk_sign.png",
            width: 66,
            height: 23,
          ),
        ),
      ],
    ),
  );
}

Widget _buildRenderPreview(
    InteractiveModeState state, Dispatch dispatch, ViewService viewService) {
  if (state.typeEnum == InteractiveTypeEnum.interactiveLive &&
      state.identityTypeEnum == InteractiveIdentityTypeEnum.audience) {
    var left = 0.0;
    var top = 0.0;
    var width = MediaQuery.of(viewService.context).size.width;
    var height = MediaQuery.of(viewService.context).size.height;

    Widget renderView;
    if (state.isClickLink) {
      renderView = _buildLiveView(
        AlivcPusherPreviewType.play,
        width,
        height,
        left: left,
        top: top,
      );
    } else {
      renderView = _buildAliPlayerView(
        width,
        height,
        left: left,
        top: top,
        onCreated: (viewId) async {
          if (state.pageDidAppear) {
            _showDialog(
              state,
              dispatch,
              viewService.context,
              pressHandle: () {
                dispatch(
                    InteractiveModeActionCreator.updateCDNPlayerViewId(viewId));
                dispatch(InteractiveModeActionCreator.startPreview());
              },
            );
            dispatch(InteractiveModeActionCreator.pageDidAppear(false));
          } else {
            dispatch(
                InteractiveModeActionCreator.updateCDNPlayerViewId(viewId));
          }
        },
      );
    }
    return renderView;
  } else {
    double left = 0.0;
    double top = 0.0;
    double width = 0.0;
    double height = 0.0;
    if (state.typeEnum == InteractiveTypeEnum.interactiveLive) {
      width = MediaQuery.of(viewService.context).size.width;
      height = MediaQuery.of(viewService.context).size.height;
    } else {
      width = MediaQuery.of(viewService.context).size.width / 2;
      height = MediaQuery.of(viewService.context).size.height / 2.1;
    }

    return _buildLiveView(AlivcPusherPreviewType.push, width, height,
        left: left, top: top, onCreated: (id) async {
      dispatch(InteractiveModeActionCreator.startPreview());
    });
  }
}

Widget _buildRenderCustomerView(
    InteractiveModeState state, Dispatch dispatch, ViewService viewService) {
  double? right;
  double? top;
  double? bottom;
  double? width;
  double? height;
  AlivcPusherPreviewType viewType = AlivcPusherPreviewType.play;

  if (state.typeEnum == InteractiveTypeEnum.interactiveLive) {
    if (state.identityTypeEnum == InteractiveIdentityTypeEnum.anchor) {
      viewType = AlivcPusherPreviewType.play;
    } else {
      viewType = AlivcPusherPreviewType.push;
    }

    right = MediaQuery.of(viewService.context).padding.right + 16;
    bottom = MediaQuery.of(viewService.context).padding.bottom + 16 * 2 + 35;
    width = 90.0;
    height = 160.0;
  } else {
    viewType = AlivcPusherPreviewType.play;
    right = MediaQuery.of(viewService.context).padding.right;
    top = 0.0;
    width = MediaQuery.of(viewService.context).size.width / 2;
    height = MediaQuery.of(viewService.context).size.height / 2.1;
  }

  if (state.isClickLink) {
    return _buildLiveView(
      viewType,
      width,
      height,
      right: right,
      top: top,
      bottom: bottom,
    );
  } else {
    return _buildLinkPlaceholerView(
      width,
      height,
      right: right,
      top: top,
      bottom: bottom,
    );
  }
}

Widget _buildLiveView(
  AlivcPusherPreviewType viewType,
  double width,
  double height, {
  double? left,
  double? right,
  double? top,
  double? bottom,
  Color? backgroundColor,
  Function? onCreated,
}) {
  return positionedView(
    color: backgroundColor ?? Colors.black,
    left: left,
    right: right,
    top: top,
    bottom: bottom,
    width: width,
    height: height,
    child: AlivcPusherPreview(
      viewType: viewType,
      onCreated: (id) {
        onCreated?.call(id);
      },
      x: 0.0,
      y: 0.0,
      width: width,
      height: height,
    ),
  );
}

Widget _buildAliPlayerView(
  double width,
  double height, {
  double? left,
  double? right,
  double? top,
  double? bottom,
  Color? backgroundColor,
  Function? onCreated,
}) {
  return positionedView(
    color: backgroundColor ?? Colors.black,
    left: left,
    right: right,
    top: top,
    bottom: bottom,
    width: width,
    height: height,
    child: AliPlayerView(
      onCreated: (viewId) async {
        onCreated?.call(viewId);
      },
      x: 0.0,
      y: 0.0,
      width: width,
      height: height,
    ),
  );
}

Widget _buildLinkPlaceholerView(
  double width,
  double height, {
  double? left,
  double? right,
  double? top,
  double? bottom,
}) {
  return positionedView(
    color: Colors.black,
    left: left,
    right: right,
    top: top,
    bottom: bottom,
    width: width,
    height: height,
    child: Image.asset(
      "assets/images/ic_interactive_unconnected_bg.png",
      width: 16,
      height: 16,
    ),
  );
}

void _showDialog(
    InteractiveModeState state, Dispatch dispatch, BuildContext context,
    {Function? pressHandle}) async {
  bool onlyUserId = state.typeEnum == InteractiveTypeEnum.interactiveLive;
  var dialog = await showDialog(
    context: context,
    builder: (BuildContext context) {
      return InteractiveMeetingInfoDialog(
        onlyUserId: onlyUserId,
        onConfirm: (userId, roomId) async {
          String _customerRoomId = "";
          if (onlyUserId) {
            if (userId == state.userId) {
              Fluttertoast.showToast(
                msg: AppLocalizations.of(context)!
                    .interactive_co_streaming_input_viewer_id_error_toast,
                gravity: ToastGravity.CENTER,
              );
              return;
            }
            _customerRoomId = state.roomId;
          } else {
            if (userId == state.userId) {
              Fluttertoast.showToast(
                msg: AppLocalizations.of(context)!
                    .interactive_battles_input_streamer_user_id_error_toast,
                gravity: ToastGravity.CENTER,
              );
              return;
            } else if (roomId == state.roomId) {
              Fluttertoast.showToast(
                msg: AppLocalizations.of(context)!
                    .interactive_battles_input_streamer_room_id_error_toast,
                gravity: ToastGravity.CENTER,
              );
              return;
            }
            _customerRoomId = roomId;
          }

          InteractiveCustomerInfo customerInfo = InteractiveCustomerInfo.create(
              userId: userId, roomId: _customerRoomId);
          dispatch(
              InteractiveModeActionCreator.updateCustomerInfo(customerInfo));
          pressHandle?.call();
        },
      );
    },
  );
}
