import 'dart:convert';
import 'dart:io';

import 'package:fish_redux/fish_redux.dart';
import 'dart:async';
import 'package:flutter/cupertino.dart' hide Action;
import 'package:flutter/widgets.dart' hide Action;
import 'package:flutter_livepush_demo/common/CommonUtils.dart';
import 'package:flutter_livepush_demo/manager/push_config_manager.dart';
import 'package:flutter_livepush_demo/page/push_config/push_config_action.dart';
import 'package:flutter_livepush_demo/page/push_config/push_config_state.dart';
import 'package:flutter_livepush_plugin/live_base.dart';
import 'package:flutter_livepush_plugin/live_pusher.dart';
import 'package:flutter_livepush_plugin/live_push_config.dart';
import 'package:flutter_livepush_demo/page/camera_push/camera_push_page.dart';
import 'package:flutter_livepush_plugin/live_push_def.dart';
import 'package:fluttertoast/fluttertoast.dart';
import 'package:flutter_livepush_demo/common/NetWorkUtils.dart';
import 'package:flutter_easyloading/flutter_easyloading.dart';
import 'package:flutter_gen/gen_l10n/app_localizations.dart';

Effect<ConfigState> buildEffect() {
  return combineEffects(<Object, Effect<ConfigState>>{
    Lifecycle.initState: _init,
    Lifecycle.dispose: _dispose,
    ConfigAction.setLivePushMode: _setLivePushMode,
    ConfigAction.setResolution: _setResolution,
    ConfigAction.setEnableAutoBitrate: _setEnableAutoBitrate,
    ConfigAction.setEnableAutoResolution: _setEnableAutoResolution,
    ConfigAction.setQualityMode: _setQualityMode,
    ConfigAction.setTargetVideoBitrate: _setTargetVideoBitrate,
    ConfigAction.setMinVideoBitrate: _setMinVideoBitrate,
    ConfigAction.setInitialVideoBitrate: _setInitialVideoBitrate,
    ConfigAction.setAudioBitrate: _setAudioBitrate,
    ConfigAction.setMinFps: _setMinFps,
    ConfigAction.setFps: _setFps,
    ConfigAction.setAudioSampleRate: _setAudioSampleRate,
    ConfigAction.setVideoEncodeGop: _setVideoEncodeGop,
    ConfigAction.setAudioEncoderProfile: _setAudioEncoderProfile,
    ConfigAction.setAudioChannel: _setAudioChannel,
    ConfigAction.setAudioEncoderMode: _setAudioEncoderMode,
    ConfigAction.setVideoEncoderMode: _setVideoEncoderMode,
    ConfigAction.setVideoHardEncoderCodec: _setVideoHardEncoderCodec,
    ConfigAction.setOpenBFrame: _setOpenBFrame,
    ConfigAction.setOrientation: _setOrientation,
    ConfigAction.setPreviewDisplayMode: _setPreviewDisplayMode,
    ConfigAction.setConnectRetryInterval: _setConnectRetryInterval,
    ConfigAction.setConnectRetryCount: _setConnectRetryCount,
    ConfigAction.setPushMirror: _setPushMirror,
    ConfigAction.setPreviewMirror: _setPreviewMirror,
    ConfigAction.setCameraType: _setCameraType,
    ConfigAction.setAutoFocus: _setAutoFocus,
    ConfigAction.setBeautyOn: _setBeautyOn,
    ConfigAction.setPauseImg: _setPauseImg,
    ConfigAction.setNetworkPoorImg: _setNetworkPoorImg,
    ConfigAction.setPushStreamingMode: _setPushStreamingMode,
    ConfigAction.openLog: _openLog,
    ConfigAction.goBackPage: _goBackPage,
    ConfigAction.clickPushPage: _clickPushPage,
  });
}

late AlivcLivePusher _alivcLivePusher;
late AlivcLivePusherConfig _alivcLivePusherConfig;

Future<void> _init(Action action, Context<ConfigState> ctx) async {
  _alivcLivePusher = AlivcLivePusher.init();
  if (ctx.state.pageType == PushConfigPageType.basic) {
    ConfigData.resetData();
    _alivcLivePusher.createConfig();
  }
  _alivcLivePusherConfig = AlivcLivePusherConfig.init();
  _setLivePusherConfig(action, ctx);
}

Future<void> _setLivePusherConfig(
    Action action, Context<ConfigState> ctx) async {
  ctx.dispatch(ConfigActionCreator.setResolution(null));
  ctx.dispatch(ConfigActionCreator.setPauseImg(true));
  ctx.dispatch(ConfigActionCreator.setNetworkPoorImg(true));
}

Future<void> _setLivePushMode(Action action, Context<ConfigState> ctx) async {
  if (action.payload != null) {
    ConfigDataUpdatePayload payload = ConfigDataUpdatePayload.create(
        ConfigAction.setLivePushMode, action.payload);
    ctx.dispatch(ConfigActionCreator.updateConfigData(payload));
  }

  AlivcLivePushMode livePushMode =
      AlivcLivePushMode.values[ctx.state.configData.livePushMode];
  _alivcLivePusherConfig.setLivePushMode(livePushMode);
}

Future<void> _setResolution(Action action, Context<ConfigState> ctx) async {
  if (action.payload != null) {
    ConfigDataUpdatePayload payload = ConfigDataUpdatePayload.create(
        ConfigAction.setResolution, action.payload);
    ctx.dispatch(ConfigActionCreator.updateConfigData(payload));
  }

  AlivcLivePushResolution resolution =
      AlivcLivePushResolution.values[ctx.state.configData.resolution.intV];
  _alivcLivePusherConfig.setResolution(resolution);

  ctx.dispatch(ConfigActionCreator.setTargetVideoBitrate(null));
  ctx.dispatch(ConfigActionCreator.setMinVideoBitrate(null));
  ctx.dispatch(ConfigActionCreator.setInitialVideoBitrate(null));
}

Future<void> _setEnableAutoBitrate(
    Action action, Context<ConfigState> ctx) async {
  if (action.payload != null) {
    ConfigDataUpdatePayload payload = ConfigDataUpdatePayload.create(
        ConfigAction.setEnableAutoBitrate, action.payload);
    ctx.dispatch(ConfigActionCreator.updateConfigData(payload));
  }

  _alivcLivePusherConfig
      .setEnableAutoBitrate(ctx.state.configData.enableAutoBitrate);
}

Future<void> _setEnableAutoResolution(
    Action action, Context<ConfigState> ctx) async {
  if (action.payload != null) {
    ConfigDataUpdatePayload payload = ConfigDataUpdatePayload.create(
        ConfigAction.setEnableAutoResolution, action.payload);
    ctx.dispatch(ConfigActionCreator.updateConfigData(payload));
  }

  _alivcLivePusherConfig
      .setEnableAutoResolution(ctx.state.configData.enableAutoResolution);
}

Future<void> _setQualityMode(Action action, Context<ConfigState> ctx) async {
  if (action.payload != null) {
    ConfigDataUpdatePayload payload = ConfigDataUpdatePayload.create(
        ConfigAction.setQualityMode, action.payload);
    ctx.dispatch(ConfigActionCreator.updateConfigData(payload));
  }

  AlivcLivePushQualityMode qualityMode =
      AlivcLivePushQualityMode.values[ctx.state.configData.qualityMode.intV];
  _alivcLivePusherConfig.setQualityMode(qualityMode);

  ctx.dispatch(ConfigActionCreator.setResolution(null));
  ctx.dispatch(ConfigActionCreator.setFps(null));
}

Future<void> _setTargetVideoBitrate(
    Action action, Context<ConfigState> ctx) async {
  if (action.payload != null) {
    ConfigDataUpdatePayload payload = ConfigDataUpdatePayload.create(
        ConfigAction.setTargetVideoBitrate, action.payload);
    ctx.dispatch(ConfigActionCreator.updateConfigData(payload));
  }

  _alivcLivePusherConfig
      .setTargetVideoBitrate(ctx.state.configData.targetVideoBitrate);
}

Future<void> _setMinVideoBitrate(
    Action action, Context<ConfigState> ctx) async {
  if (action.payload != null) {
    ConfigDataUpdatePayload payload = ConfigDataUpdatePayload.create(
        ConfigAction.setMinVideoBitrate, action.payload);
    ctx.dispatch(ConfigActionCreator.updateConfigData(payload));
  }

  _alivcLivePusherConfig
      .setMinVideoBitrate(ctx.state.configData.minVideoBitrate);
}

Future<void> _setInitialVideoBitrate(
    Action action, Context<ConfigState> ctx) async {
  if (action.payload != null) {
    ConfigDataUpdatePayload payload = ConfigDataUpdatePayload.create(
        ConfigAction.setInitialVideoBitrate, action.payload);
    ctx.dispatch(ConfigActionCreator.updateConfigData(payload));
  }

  _alivcLivePusherConfig
      .setInitialVideoBitrate(ctx.state.configData.initialVideoBitrate);
}

Future<void> _setAudioBitrate(Action action, Context<ConfigState> ctx) async {
  if (action.payload != null) {
    ConfigDataUpdatePayload payload = ConfigDataUpdatePayload.create(
        ConfigAction.setAudioBitrate, action.payload);
    ctx.dispatch(ConfigActionCreator.updateConfigData(payload));
  }

  _alivcLivePusherConfig.setAudioBitrate(ctx.state.configData.audioBitrate);
}

Future<void> _setMinFps(Action action, Context<ConfigState> ctx) async {
  if (action.payload != null) {
    ConfigDataUpdatePayload payload =
        ConfigDataUpdatePayload.create(ConfigAction.setMinFps, action.payload);
    ctx.dispatch(ConfigActionCreator.updateConfigData(payload));
  }

  AlivcLivePushFPS minFps =
      AlivcLivePushFPS.values[ctx.state.configData.minFps.intV];
  _alivcLivePusherConfig.setMinFps(minFps);
}

Future<void> _setFps(Action action, Context<ConfigState> ctx) async {
  if (action.payload != null) {
    ConfigDataUpdatePayload payload =
        ConfigDataUpdatePayload.create(ConfigAction.setFps, action.payload);
    ctx.dispatch(ConfigActionCreator.updateConfigData(payload));
  }

  AlivcLivePushFPS fps = AlivcLivePushFPS.values[ctx.state.configData.fps.intV];
  _alivcLivePusherConfig.setFps(fps);
}

Future<void> _setAudioSampleRate(
    Action action, Context<ConfigState> ctx) async {
  if (action.payload != null) {
    ConfigDataUpdatePayload payload = ConfigDataUpdatePayload.create(
        ConfigAction.setAudioSampleRate, action.payload);
    ctx.dispatch(ConfigActionCreator.updateConfigData(payload));
  }

  AlivcLivePushAudioSampleRate audioSampleRate = AlivcLivePushAudioSampleRate
      .values[ctx.state.configData.audioSampleRate.intV];
  _alivcLivePusherConfig.setAudioSampleRate(audioSampleRate);
}

Future<void> _setVideoEncodeGop(Action action, Context<ConfigState> ctx) async {
  if (action.payload != null) {
    ConfigDataUpdatePayload payload = ConfigDataUpdatePayload.create(
        ConfigAction.setVideoEncodeGop, action.payload);
    ctx.dispatch(ConfigActionCreator.updateConfigData(payload));
  }

  AlivcLivePushVideoEncodeGOP videoEncodeGOP =
      VideoEncodeGOPData.convertEnumValue(ctx.state.configData.videoEncodeGop);
  _alivcLivePusherConfig.setVideoEncodeGop(videoEncodeGOP);
}

Future<void> _setAudioEncoderProfile(
    Action action, Context<ConfigState> ctx) async {
  if (action.payload != null) {
    ConfigDataUpdatePayload payload = ConfigDataUpdatePayload.create(
        ConfigAction.setAudioEncoderProfile, action.payload);
    ctx.dispatch(ConfigActionCreator.updateConfigData(payload));
  }

  AlivcLivePushAudioEncoderProfile audioEncoderProfile =
      AlivcLivePushAudioEncoderProfile
          .values[ctx.state.configData.audioEncoderProfile.intV];
  _alivcLivePusherConfig.setAudioEncoderProfile(audioEncoderProfile);
}

Future<void> _setAudioChannel(Action action, Context<ConfigState> ctx) async {
  if (action.payload != null) {
    ConfigDataUpdatePayload payload = ConfigDataUpdatePayload.create(
        ConfigAction.setAudioChannel, action.payload);
    ctx.dispatch(ConfigActionCreator.updateConfigData(payload));
  }

  AlivcLivePushAudioChannel audioChannel =
      AlivcLivePushAudioChannel.values[ctx.state.configData.audioChannel.intV];
  _alivcLivePusherConfig.setAudioChannel(audioChannel);
}

Future<void> _setAudioEncoderMode(
    Action action, Context<ConfigState> ctx) async {
  if (action.payload != null) {
    ConfigDataUpdatePayload payload = ConfigDataUpdatePayload.create(
        ConfigAction.setAudioEncoderMode, action.payload);
    ctx.dispatch(ConfigActionCreator.updateConfigData(payload));
  }

  AlivcLivePushAudioEncoderMode audioEncoderMode = AlivcLivePushAudioEncoderMode
      .values[ctx.state.configData.audioEncoderMode];
  _alivcLivePusherConfig.setAudioEncoderMode(audioEncoderMode);
}

Future<void> _setVideoEncoderMode(
    Action action, Context<ConfigState> ctx) async {
  if (action.payload != null) {
    ConfigDataUpdatePayload payload = ConfigDataUpdatePayload.create(
        ConfigAction.setVideoEncoderMode, action.payload);
    ctx.dispatch(ConfigActionCreator.updateConfigData(payload));
  }

  AlivcLivePushVideoEncoderMode videoEncoderMode = AlivcLivePushVideoEncoderMode
      .values[ctx.state.configData.videoEncoderMode];
  _alivcLivePusherConfig.setVideoEncoderMode(videoEncoderMode);
}

Future<void> _setVideoHardEncoderCodec(
    Action action, Context<ConfigState> ctx) async {
  if (action.payload != null) {
    ConfigDataUpdatePayload payload = ConfigDataUpdatePayload.create(
        ConfigAction.setVideoHardEncoderCodec, action.payload);
    ctx.dispatch(ConfigActionCreator.updateConfigData(payload));
  }

  AlivcLivePushVideoEncoderModeHardCodec videoEncoderModeHardCodec =
      AlivcLivePushVideoEncoderModeHardCodec
          .values[ctx.state.configData.videoHardEncoderCodec.intV];
  _alivcLivePusherConfig.setVideoHardEncoderCodec(videoEncoderModeHardCodec);
}

Future<void> _setOpenBFrame(Action action, Context<ConfigState> ctx) async {
  if (action.payload != null) {
    ConfigDataUpdatePayload payload = ConfigDataUpdatePayload.create(
        ConfigAction.setOpenBFrame, action.payload);
    ctx.dispatch(ConfigActionCreator.updateConfigData(payload));
  }

  _alivcLivePusherConfig.setOpenBFrame(ctx.state.configData.openBFrame);
}

Future<void> _setOrientation(Action action, Context<ConfigState> ctx) async {
  if (action.payload != null) {
    ConfigDataUpdatePayload payload = ConfigDataUpdatePayload.create(
        ConfigAction.setOrientation, action.payload);
    ctx.dispatch(ConfigActionCreator.updateConfigData(payload));
  }

  AlivcLivePushOrientation orientation =
      AlivcLivePushOrientation.values[ctx.state.configData.orientation.intV];
  _alivcLivePusherConfig.setOrientation(orientation);

  if (ctx.state.configData.openPauseImg == true) {
    ctx.dispatch(ConfigActionCreator.setPauseImg(true));
  }
  if (ctx.state.configData.openNetworkPoorImg == true) {
    ctx.dispatch(ConfigActionCreator.setNetworkPoorImg(true));
  }
}

Future<void> _setPreviewDisplayMode(
    Action action, Context<ConfigState> ctx) async {
  if (action.payload != null) {
    ConfigDataUpdatePayload payload = ConfigDataUpdatePayload.create(
        ConfigAction.setPreviewDisplayMode, action.payload);
    ctx.dispatch(ConfigActionCreator.updateConfigData(payload));
  }

  AlivcPusherPreviewDisplayMode previewDisplayMode =
      AlivcPusherPreviewDisplayMode
          .values[ctx.state.configData.previewDisplayMode.intV];
  _alivcLivePusherConfig.setPreviewDisplayMode(previewDisplayMode);
}

Future<void> _setConnectRetryInterval(
    Action action, Context<ConfigState> ctx) async {
  if (action.payload != null) {
    ConfigDataUpdatePayload payload = ConfigDataUpdatePayload.create(
        ConfigAction.setConnectRetryInterval, action.payload);
    ctx.dispatch(ConfigActionCreator.updateConfigData(payload));
  }

  _alivcLivePusherConfig
      .setConnectRetryInterval(ctx.state.configData.connectRetryInterval);
}

Future<void> _setConnectRetryCount(
    Action action, Context<ConfigState> ctx) async {
  if (action.payload != null) {
    ConfigDataUpdatePayload payload = ConfigDataUpdatePayload.create(
        ConfigAction.setConnectRetryCount, action.payload);
    ctx.dispatch(ConfigActionCreator.updateConfigData(payload));
  }

  _alivcLivePusherConfig
      .setConnectRetryCount(ctx.state.configData.connectRetryCount);
}

Future<void> _setPushMirror(Action action, Context<ConfigState> ctx) async {
  if (action.payload != null) {
    ConfigDataUpdatePayload payload = ConfigDataUpdatePayload.create(
        ConfigAction.setPushMirror, action.payload);
    ctx.dispatch(ConfigActionCreator.updateConfigData(payload));
  }

  _alivcLivePusherConfig.setPushMirror(ctx.state.configData.pushMirror);
}

Future<void> _setPreviewMirror(Action action, Context<ConfigState> ctx) async {
  if (action.payload != null) {
    ConfigDataUpdatePayload payload = ConfigDataUpdatePayload.create(
        ConfigAction.setPreviewMirror, action.payload);
    ctx.dispatch(ConfigActionCreator.updateConfigData(payload));
  }

  _alivcLivePusherConfig.setPreviewMirror(ctx.state.configData.previewMirror);
}

Future<void> _setCameraType(Action action, Context<ConfigState> ctx) async {
  if (action.payload != null) {
    ConfigDataUpdatePayload payload = ConfigDataUpdatePayload.create(
        ConfigAction.setCameraType, action.payload);
    ctx.dispatch(ConfigActionCreator.updateConfigData(payload));
  }

  AlivcLivePushCameraType cameraType =
      AlivcLivePushCameraType.values[ctx.state.configData.cameraType];
  _alivcLivePusherConfig.setCameraType(cameraType);
}

Future<void> _setAutoFocus(Action action, Context<ConfigState> ctx) async {
  if (action.payload != null) {
    ConfigDataUpdatePayload payload = ConfigDataUpdatePayload.create(
        ConfigAction.setAutoFocus, action.payload);
    ctx.dispatch(ConfigActionCreator.updateConfigData(payload));
  }

  _alivcLivePusherConfig.setAutoFocus(ctx.state.configData.autoFocus);
}

Future<void> _setBeautyOn(Action action, Context<ConfigState> ctx) async {
  if (action.payload != null) {
    ConfigDataUpdatePayload payload = ConfigDataUpdatePayload.create(
        ConfigAction.setBeautyOn, action.payload);
    ctx.dispatch(ConfigActionCreator.updateConfigData(payload));
  }
}

Future<void> _setPauseImg(Action action, Context<ConfigState> ctx) async {
  if (action.payload != null) {
    ConfigDataUpdatePayload payload = ConfigDataUpdatePayload.create(
        ConfigAction.setPauseImg, action.payload);
    ctx.dispatch(ConfigActionCreator.updateConfigData(payload));
  }

  if (ctx.state.configData.openPauseImg == true) {
    String imageName = "";
    if (ctx.state.configData.orientation.intV == 0) {
      imageName = "assets/images/pauseImg.png";
    } else {
      imageName = "assets/images/pauseImg_land.png";
    }

    DirType dirType;
    if (Platform.isAndroid) {
      dirType = DirType.externalFile;
    } else {
      dirType = DirType.document;
    }
    var split = imageName.split("/");
    CommomUtils.getSaveDir(dirType, split[2])
        .then((value) => _alivcLivePusherConfig.setPauseImg(value.path));
  } else {
    _alivcLivePusherConfig.setPauseImg("");
  }
}

Future<void> _setNetworkPoorImg(Action action, Context<ConfigState> ctx) async {
  if (action.payload != null) {
    ConfigDataUpdatePayload payload = ConfigDataUpdatePayload.create(
        ConfigAction.setNetworkPoorImg, action.payload);
    ctx.dispatch(ConfigActionCreator.updateConfigData(payload));
  }

  if (ctx.state.configData.openNetworkPoorImg == true) {
    String imageName = "";
    if (ctx.state.configData.orientation.intV == 0) {
      imageName = "assets/images/poorNetworkImg.png";
    } else {
      imageName = "assets/images/poorNetworkImg_land.png";
    }

    DirType dirType;
    if (Platform.isAndroid) {
      dirType = DirType.externalFile;
    } else {
      dirType = DirType.document;
    }
    var split = imageName.split("/");
    CommomUtils.getSaveDir(dirType, split[2])
        .then((value) => _alivcLivePusherConfig.setNetworkPoorImg(value.path));
  } else {
    _alivcLivePusherConfig.setNetworkPoorImg("");
  }
}

Future<void> _setPushStreamingMode(
    Action action, Context<ConfigState> ctx) async {
  if (action.payload != null) {
    ConfigDataUpdatePayload payload = ConfigDataUpdatePayload.create(
        ConfigAction.setPushStreamingMode, action.payload);
    ctx.dispatch(ConfigActionCreator.updateConfigData(payload));
  }

  _alivcLivePusherConfig
      .setAudioOnly(ctx.state.configData.pushStreamingMode.intV == 1);
  _alivcLivePusherConfig
      .setVideoOnly(ctx.state.configData.pushStreamingMode.intV == 2);
}

Future<void> _openLog(Action action, Context<ConfigState> ctx) async {
  if (action.payload != null) {
    ConfigDataUpdatePayload payload =
        ConfigDataUpdatePayload.create(ConfigAction.openLog, action.payload);
    ctx.dispatch(ConfigActionCreator.updateConfigData(payload));
  }

  if (ctx.state.configData.openLog == true) {
    String saveLogDir = "";
    String saveAudioDir = ExternalStreamData.downloadDir +
        "/" +
        ExternalStreamData.audioResourceName;
    if (Platform.isAndroid) {
      saveLogDir = await CommomUtils.getSystemPath(DirType.externalFile);
    } else {
      saveLogDir = await CommomUtils.getSystemPath(DirType.document);
    }
    AlivcBase.setLogPath(saveLogDir, saveLogMaxPartFileSizeInKB);
    AlivcBase.setLogLevel(AlivcLivePushLogLevel.debug);
    AlivcBase.setConsoleEnable(true);
  } else {
    AlivcBase.setLogLevel(AlivcLivePushLogLevel.none);
    AlivcBase.setConsoleEnable(false);
  }
}

Future<void> _goBackPage(Action action, Context<ConfigState> ctx) async {
  FocusScope.of(ctx.context).unfocus();
  Navigator.of(ctx.context).pop();

  if (ctx.state.onSettingConfigAction != null) {
    ctx.state.onSettingConfigAction!(_alivcLivePusherConfig);
  }
}

Future<void> _clickPushPage(Action action, Context<ConfigState> ctx) async {
  if (ctx.state.pageType == PushConfigPageType.basic) {
    if (action.payload == null || action.payload.length == 0) {
      Fluttertoast.showToast(
          msg: AppLocalizations.of(ctx.context)!.config_enter_url_tip,
          gravity: ToastGravity.CENTER);
    } else {
      if (ctx.state.configData.fps.intV < ctx.state.configData.minFps.intV) {
        Fluttertoast.showToast(
            msg: AppLocalizations.of(ctx.context)!.config_fps_tip,
            gravity: ToastGravity.CENTER);
        return;
      }

      FocusScope.of(ctx.context).unfocus();
      Navigator.of(ctx.context).push(CupertinoPageRoute(
          builder: (_) => CameraPushPage().buildPage({
                pushURLKey: action.payload,
                pushConfigKey: _alivcLivePusherConfig,
                pushBeautyOnKey: ctx.state.configData.beautyOn,
              })));
    }
  } else {
    FocusScope.of(ctx.context).unfocus();
    Navigator.of(ctx.context).pop();

    if (ctx.state.onSettingConfigAction != null) {
      ctx.state.onSettingConfigAction!(_alivcLivePusherConfig);
    }
  }
}

Future<void> _dispose(Action action, Context<ConfigState> ctx) async {}
