import 'package:fish_redux/fish_redux.dart';
import 'package:flutter/material.dart';
import 'package:flutter_livepush_demo/common/CommonUtils.dart';
import 'package:flutter_livepush_demo/models/music_info.dart';
import 'package:flutter_livepush_demo/page/camera_push/camera_push_action.dart';
import 'package:flutter_livepush_demo/page/camera_push/camera_push_state.dart';
import 'package:flutter_livepush_demo/view/config_cell.dart';
import 'package:flutter_gen/gen_l10n/app_localizations.dart';

String _muteAssets = "assets/images/ic_mute_off.png";
String _playAssets = "assets/images/ic_music_pause.png";
String _loopAssets = "assets/images/ic_loop_off.png";

Widget bottomSheetMusic(
    CameraPushState state, ViewService viewService, Dispatch dispatch) {
  return Container(
    clipBehavior: Clip.antiAlias,
    decoration: const BoxDecoration(
      color: Color(0xff1C1D22),
      borderRadius: BorderRadius.only(
        topLeft: Radius.circular(20.0),
        topRight: Radius.circular(20.0),
      ),
    ),
    height: MediaQuery.of(viewService.context).size.height - 50.0,
    padding: EdgeInsets.only(
      left: MediaQuery.of(viewService.context).padding.left,
      right: MediaQuery.of(viewService.context).padding.right,
    ),
    child: Column(
      children: [
        _musicHeader(viewService.context, dispatch),
        const Divider(height: 1.0),
        ConfigCell(
          type: ConfigCellType.switchSelect,
          title: AppLocalizations.of(viewService.context)!
              .camerapush_music_enable_in_ear_monitoring,
          defaultValue: state.bgmState.isOpenBGMEarsBack,
          action: (value) {
            dispatch(
              CameraPushActionCreator.changeEars(value),
            );
          },
        ),
        ConfigCell(
          type: ConfigCellType.switchSelect,
          title: AppLocalizations.of(viewService.context)!
              .camerapush_music_noise_reduction,
          defaultValue: state.bgmState.isOpenAudioDenoise,
          action: (value) {
            dispatch(
              CameraPushActionCreator.changeAudioDenoise(value),
            );
          },
        ),
        Expanded(
          child: ListView.builder(
            itemBuilder: (BuildContext context, int index) {
              if (index == 0) {
                return _buildMusicListControlItem(state, context, dispatch);
              } else {
                return _buildMusicListNormalItem(
                    state, context, musicList, index - 1, dispatch);
              }
            },
            itemCount: musicList.length + 1,
          ),
        ),
      ],
    ),
  );
}

Widget _musicHeader(context, Dispatch dispatch) {
  return SizedBox(
    height: 40,
    child: Stack(
      textDirection: TextDirection.rtl,
      children: [
        Center(
          child: Row(
            mainAxisAlignment: MainAxisAlignment.spaceAround,
            children: [
              InkWell(
                onTap: () {
                  dispatch(
                    CameraPushActionCreator.hideMusic(),
                  );
                },
                child: Text(
                  AppLocalizations.of(context)!.config_cell_cancel_button,
                  style: const TextStyle(
                    fontSize: 15,
                    color: Color(0xFFFCFCFD),
                  ),
                ),
              ),
              Text(
                AppLocalizations.of(context)!.camerapush_music,
                style: const TextStyle(
                  fontSize: 15,
                  color: Color(0xFFFCFCFD),
                ),
              ),
              InkWell(
                onTap: () {
                  dispatch(
                    CameraPushActionCreator.hideMusic(),
                  );
                },
                child: Text(
                  AppLocalizations.of(context)!.config_cell_ok_button,
                  style: const TextStyle(
                    fontSize: 15,
                    color: Color(0xFF4DCFE1),
                  ),
                ),
              ),
            ],
          ),
        ),
      ],
    ),
  );
}

bool bgmMute = false;

Widget _buildMusicListControlItem(
    CameraPushState state, BuildContext context, Dispatch dispatch) {
  if (state.bgmState.isBGMMute) {
    _muteAssets = "assets/images/ic_mute_on.png";
  } else {
    _muteAssets = "assets/images/ic_mute_off.png";
  }
  if (state.bgmState.isBGMPlaying) {
    _playAssets = "assets/images/ic_music_play.png";
  } else {
    _playAssets = "assets/images/ic_music_pause.png";
  }
  if (state.bgmState.isBGMLoop) {
    _loopAssets = "assets/images/ic_loop_on.png";
  } else {
    _loopAssets = "assets/images/ic_loop_off.png";
  }
  return Container(
    margin: const EdgeInsets.all(20),
    padding: const EdgeInsets.all(10),
    clipBehavior: Clip.antiAlias,
    decoration: const BoxDecoration(
      color: Color(0xFF1C1D22),
      borderRadius: BorderRadius.all(
        Radius.circular(20.0),
      ),
    ),
    child: Column(
      crossAxisAlignment: CrossAxisAlignment.start,
      children: [
        Row(
          mainAxisAlignment: MainAxisAlignment.spaceBetween,
          children: [
            Text(
              AppLocalizations.of(context)!.camerapush_music_item(
                  musicList[state.bgmState.bgmSelectedIndex].name),
              style: const TextStyle(
                fontSize: 16,
                color: Color(0xFFFCFCFD),
              ),
            ),
            Image.asset(
              "assets/images/ic_ugsv_music_yinfu.png",
              width: 16,
              height: 16,
            ),
          ],
        ),
        const SizedBox(height: 20.0),
        Slider(
          min: 0.0,
          max: state.bgmState.bgmDuration.toDouble(),
          value: state.bgmState.bgmProgress.toDouble(),
          onChanged: (double value) {},
        ),
        const SizedBox(height: 20.0),
        Row(
          mainAxisAlignment: MainAxisAlignment.spaceBetween,
          children: [
            GestureDetector(
              onTap: (() {
                dispatch(CameraPushActionCreator.changeMute(
                    !state.bgmState.isBGMMute));
              }),
              child: Image.asset(
                _muteAssets,
                width: 28,
                height: 28,
              ),
            ),
            GestureDetector(
              onTap: (() {
                dispatch(
                  CameraPushActionCreator.changePlayState(
                      !state.bgmState.isBGMPlaying),
                );
              }),
              child: Image.asset(
                _playAssets,
                width: 28,
                height: 28,
              ),
            ),
            GestureDetector(
              onTap: (() {
                dispatch(
                  CameraPushActionCreator.changeLoop(!state.bgmState.isBGMLoop),
                );
              }),
              child: Image.asset(
                _loopAssets,
                width: 28,
                height: 28,
              ),
            ),
          ],
        ),
        const SizedBox(height: 20.0),
        Text(
          AppLocalizations.of(context)!.camerapush_music_accompaniment_volume,
          style: const TextStyle(
            fontSize: 15,
            color: Color(0xFFFCFCFD),
          ),
        ),
        Slider(
            min: 0,
            max: 100,
            value: state.bgmState.bgmVolume.toDouble(),
            onChanged: (value) {
              dispatch(
                CameraPushActionCreator.changeBGMVolume(value.toInt()),
              );
            }),
        Text(
          AppLocalizations.of(context)!.camerapush_music_vocal_volume,
          style: const TextStyle(
            fontSize: 15,
            color: Color(0xFFFCFCFD),
          ),
        ),
        Slider(
          min: 0,
          max: 100,
          value: state.bgmState.bgmCaptureVolume.toDouble(),
          onChanged: (value) {
            dispatch(
              CameraPushActionCreator.changeCaptureVolume(value.toInt()),
            );
          },
        ),
      ],
    ),
  );
}

Widget _buildMusicListNormalItem(CameraPushState state, BuildContext context,
    List<MusicInfo> datas, int index, Dispatch dispatch) {
  Widget selectImg = const SizedBox();
  if (state.bgmState.bgmSelectedIndex == index) {
    selectImg = Image.asset(
      "assets/images/ic_check.png",
      width: 22,
      height: 22,
    );
  } else {
    selectImg = const SizedBox();
  }
  return GestureDetector(
    onTap: () {
      MusicInfo selectMusicInfo = datas[index].clone();
      dispatch(
        CameraPushActionCreator.changeBGMSource(selectMusicInfo),
      );
    },
    child: SizedBox(
      height: 45,
      child: Column(
        children: [
          Row(
            mainAxisAlignment: MainAxisAlignment.spaceBetween,
            children: [
              Text(
                AppLocalizations.of(context)!
                    .camerapush_music_item(datas[index].name),
                style: const TextStyle(
                  fontSize: 15,
                  color: Color(0xFFFCFCFD),
                ),
              ),
              selectImg,
            ],
          ),
          const Divider(height: 5.0),
        ],
      ),
    ),
  );
}
