import 'package:fish_redux/fish_redux.dart';
import 'package:flutter/material.dart';
import 'package:flutter_livepush_demo/manager/push_config_manager.dart';
import 'package:flutter_livepush_demo/page/camera_push/camera_push_action.dart';
import 'package:flutter_livepush_demo/page/camera_push/camera_push_state.dart';
import 'package:flutter_livepush_demo/view/config_cell.dart';
import 'package:flutter_gen/gen_l10n/app_localizations.dart';

Widget showSetting(context, CameraPushState state, Dispatch dispatch) {
  return Container(
    clipBehavior: Clip.antiAlias,
    decoration: const BoxDecoration(
      color: Color(0xff1C1D22),
      borderRadius: BorderRadius.only(
        topLeft: Radius.circular(20.0),
        topRight: Radius.circular(20.0),
      ),
    ),
    height: MediaQuery.of(context).size.height - 200,
    padding: EdgeInsets.only(
      left: MediaQuery.of(context).padding.left,
      right: MediaQuery.of(context).padding.right,
    ),
    child: Column(
      mainAxisSize: MainAxisSize.min,
      children: [
        _showSettingHead(context, state, dispatch),
        const Divider(height: 1.0),
        Expanded(
          child: _showSettingContent(context, state, dispatch),
        ),
      ],
    ),
  );
}

Widget _showSettingHead(context, CameraPushState state, Dispatch dispatch) {
  return SizedBox(
    height: 40,
    child: Stack(
      textDirection: TextDirection.rtl,
      children: [
        Center(
          child: Row(
            mainAxisAlignment: MainAxisAlignment.spaceAround,
            children: [
              InkWell(
                onTap: (() => dispatch(
                      CameraPushActionCreator.hideMoreSetting(),
                    )),
                child: Text(
                  AppLocalizations.of(context)!.config_cell_cancel_button,
                  style: const TextStyle(
                    fontSize: 15,
                    color: Color(0xFFFCFCFD),
                  ),
                ),
              ),
              Text(
                AppLocalizations.of(context)!.camerapush_more_setting,
                style: const TextStyle(
                  fontSize: 15,
                  color: Color(0xFFFCFCFD),
                ),
              ),
              InkWell(
                onTap: () {
                  dispatch(
                    CameraPushActionCreator.setTargetVideoBitrate(
                        state.pushSettingState.videoTargetBitrate),
                  );
                  dispatch(
                    CameraPushActionCreator.setMinVideoBitrate(
                        state.pushSettingState.videoMinBitrate),
                  );
                  dispatch(CameraPushActionCreator.hideMoreSetting());
                },
                child: Text(
                  AppLocalizations.of(context)!.config_cell_ok_button,
                  style: const TextStyle(
                    fontSize: 15,
                    color: Color(0xFF4DCFE1),
                  ),
                ),
              )
            ],
          ),
        ),
      ],
    ),
  );
}

Widget _showSettingContent(context, CameraPushState state, Dispatch dispatch) {
  return ListView(
    children: [
      ConfigCell(
        type: ConfigCellType.input,
        title: AppLocalizations.of(context)!.config_target_bitrate,
        infoUnit: "/Kbps",
        defaultValue: state.pushSettingState.videoTargetBitrate.toString(),
        action: (value) {
          dispatch(
            CameraPushActionCreator.setTargetVideoBitrate(int.parse(value)),
          );
        },
      ),
      ConfigCell(
        type: ConfigCellType.input,
        title: AppLocalizations.of(context)!.config_min_bitrate,
        infoUnit: "/Kbps",
        defaultValue: state.pushSettingState.videoMinBitrate.toString(),
        action: (value) {
          dispatch(
            CameraPushActionCreator.setMinVideoBitrate(int.parse(value)),
          );
        },
      ),
      ConfigCell(
        type: ConfigCellType.switchSelect,
        title: AppLocalizations.of(context)!.config_push_mirror,
        defaultValue: state.pushSettingState.isOpenPushMirror,
        action: (value) {
          dispatch(
            CameraPushActionCreator.changePushMirror(value),
          );
        },
      ),
      ConfigCell(
        type: ConfigCellType.switchSelect,
        title: AppLocalizations.of(context)!.config_preview_mirror,
        defaultValue: state.pushSettingState.isOpenPreviewMirror,
        action: (value) {
          dispatch(
            CameraPushActionCreator.changePreviewMirror(value),
          );
        },
      ),
      ConfigCell(
        type: ConfigCellType.pickerSelect,
        title: AppLocalizations.of(context)!.config_preview_display_mode,
        pickerPanelTexts: previewDisplayModeItems.values.toList(),
        defaultValue: previewDisplayModeItems.values
            .toList()[state.pushSettingState.previewModelIndex],
        action: (value) {
          dispatch(
            CameraPushActionCreator.setPreviewDisplayMode(value.index),
          );
        },
      ),
    ],
  );
}
